####################################################################
# (C) Copyright 2013 Hewlett-Packard Development Company, L.P.
# @(#) Serviceguard cluster verification script
# @(#) Product Name              :  HP Serviceguard
# @(#) Product Version           :  A.12.10.00
# @(#) Patch Name                :  
#
# *** Note: This file MUST NOT be edited. *****
#
# Any changes made to it will be overwritten when you upgrade to the
# next release of HP Serviceguard.
#
# Changing this file may lead to failures during Cluster verification
#
# DESCRIPTION: This script is invoked by Serviceguard during Cluster 
#              verification to perform a set of cluster level checks.
#              This master script maintains a list of shell scripts which 
#              will be used for validations. The actions performed by
#              this script are as follows.
#
#              - calls each script with "-o validate" option on all 
#                the nodes in the cluster which should return the parameter
#                and the value(to be compared) in the form of "a=b"
#
#              - Write the data obtained from nodes to an env file
#                as "SG_<node_name>_a=b"
#
#              - calls the same script with "-o consolidate" on the 
#                local node/remote node to validate these parameter values.
#             
# ARGUMENTS:   
#         -n   node which should be considered for validation
#         
#         -u   node which is unreachable and should not be considered 
#              for validation. Will populate empty strings for the 
#              env vars for this node
#
# EXAMPLE:
#         cluster_master_script.sh -n abc -n xyz -u ab1  
#
# RETURNS:     zero       if all validations are successful
#              non-zero   if any of the scripts returns non-zero value
#
####################################################################

. ${SGCONFFILE:=/etc/cmcluster.conf}
. ${SGUTILS:=$SGCONF/scripts/cluster_master_script_utils.sh}

declare -a validate_scripts
typeset -i index=0
typeset -i ind=0
typeset nodes
typeset unreach_nodes
typeset unavail_nodes
typeset -i aggregate_exit_val=0
typeset -i ret=0
typeset str=""
typeset val=""
typeset src_file="$SGCONF/cmscripts2verify"
typeset env_file="/var/tmp/master_cluster_script.env"
typeset remote_node=""
typeset string=""
typeset -i is_avail_on_local_node
typeset -i timeout=600
typeset -i debug=0
typeset logfile=""
typeset validate_cmd="-o validate"
typeset consolidate_cmd="-o consolidate"

while getopts "n:u:O:D" opt;do
case "$opt" in
    n)
        nodes[$index]=$OPTARG
        (( index = index + 1 ))
        ;;
    u)
        unreach_nodes[$ind]=$OPTARG
        (( ind = ind + 1 ))
        ;;
    D)
        debug=1
        ;;
    O)
        logfile=$OPTARG
        ;;
    *)
        echo "INFO: Unknown operation - $1"
        exit 1
        ;;
esac
done

#######################################################
# This function writes the values obtained from the
# validation scripts by prefixing it with SG_<node_name>
# to the env file
#######################################################

function write_to_file
{
    typeset nd_name=$1
    typeset -i empty=$2
    typeset output=$3
    typeset value
    OLDIFS=$IFS
    IFS=$'\n'
    for line in $output
    do
        value="SG_${nd_name}_$line"
        if [ $empty -eq 1 ]; then
            value=${value/=*/=\"\"}
        fi
        echo "$value" >> $env_file
    done 
    IFS=$OLDIFS
}

function construct_unavail_nodes
{
    declare -a array=$@

    for nd in ${array[*]}
    do
        write_to_file $nd 1 $string
    done
}

function construct_node_names
{
    typeset -i idx=0
    for nd in ${nodes[*]}
    do
        value="SG_NODES[$idx]=$nd"
        echo "$value" >> $env_file
        (( idx = idx + 1 ))
    done
    
    idx=0
    for nd in ${unreach_nodes[*]}
    do
        value="SG_UNREACHABLE_NODES[$idx]=$nd"
        echo "$value" >> $env_file
        (( idx = idx + 1 ))
    done

}

##############################################################
# This function reads the file $SGCONF/files2check to get the
# list of scripts to be used for validation
##############################################################

function construct_validate_scripts
{
    typeset -i i=0
    typeset output=$(cat $src_file | sed -e 's/^[ \t]*//' -e '/^#/d' -e '/^$/d')
    OLDIFS=$IFS
    IFS=$'\n'

    for line in $output
    do
        validate_scripts[$i]=$line
        (( i = i + 1 ))
    done
    IFS=$OLDIFS
}

#############################
#       MAIN ROUTINE
#############################

HOST=$(uname -n | cut -d . -f 1)
ind=0

if [ $debug -eq 1 ]; then
   validate_cmd="$validate_cmd -D"
   consolidate_cmd="$consolidate_cmd -D"
   export "SG_LOG_DEBUG=1"                   
fi

if [ ! -z $logfile ]; then
   validate_cmd="$validate_cmd -O $logfile"
   consolidate_cmd="$consolidate_cmd -O $logfile"
   export "SG_LOG_FILE=$logfile"
fi

construct_validate_scripts

sg_log_cv 1 "Scripts to be used for validation: ${validate_scripts[*]}"
sg_log_cv 1 "Nodes : ${nodes[*]}"

for script in ${validate_scripts[*]}
do
    is_avail_on_local_node=1

    if [ ! -f $script ]; then
        is_avail_on_local_node=0
    fi
    
    rm -rf $env_file

    construct_node_names

    for node in ${nodes[*]}
    do
        sg_log_cv 1 "Running $script $validate_cmd on $node"

        if [[ $HOST == $node ]]; then
            val=$($script $validate_cmd 2>&1)
            ret=$?
        else
            val=$($SGSBIN/cmexec $node -t $timeout $script $validate_cmd 2>&1)
            ret=$? 
        fi

        if [ $ret -ne 0 ]; then
            unavail_nodes[$ind]=$node
            (( ind = ind + 1 ))
            if [[ $val == *"No such file or directory"* ]]; then
                continue  
            fi
            echo "WARNING: Unable to get information on $node"
            echo "         Script $script returned $val"
            continue
        fi 

        sg_log_cv 1 "Output: $val"

        if [ -z $remote_node ];then
            remote_node=$node
            string=$val
        fi
        write_to_file $node 0 $val
    done

    if [[ "$string" != "" ]]; then

        # Populate entries with empty string values for the
        # unreachable nodes and the nodes where the script was
        # non-existent

        construct_unavail_nodes ${unreach_nodes[@]}

        construct_unavail_nodes ${unavail_nodes[@]}

        content=$(cat $env_file)
        sg_log_cv 1 "Environment file contains:"
        sg_log_cv 1 "$content"

        if [ $is_avail_on_local_node -eq 1 ]; then
            sg_log_cv 1 "Running $script $consolidate_cmd on $HOST"
            $script $consolidate_cmd
            ret=$?
        else
            # Validation script not available on local node. 
            # Copy the env file to the remote node and run the
            # script on the remote node
            sg_log_cv 1 "$script not available on $HOST"
            sg_log_cv 1 "Running $script $consolidate_cmd on $remote_node"
 
            $SGSBIN/cmcp $env_file $remote_node:$env_file
            $SGSBIN/cmexec $remote_node -t $timeout $script $consolidate_cmd
            ret=$?

            # Delete the env file on the remote node
            $SGSBIN/cmexec $remote_node -t $timeout rm -rf $env_file
        fi

        if [ $ret -ne 0 ];then
            sg_log_cv 1 "Script $script returned: $ret"
            aggregate_exit_val=$ret
        fi
    fi
   
done

rm -rf $env_file

exit $aggregate_exit_val

