# @(#) Serviceguard package control script

####################################################################
# (C) Copyright 2006-2012 Hewlett-Packard Development Company, L.P.
# @(#) Serviceguard UPCC Package Configuration Script.
# @(#) Product Name                :  HP Serviceguard
# @(#) Product Version             :  A.12.10.00
# @(#) Patch Name                  :  
#
# *** Note: This file MUST NOT be edited. *****
#
# Any changes made to it will be overwritten when you upgrade to the
# next release of HP Serviceguard.
#
# Changing this file may lead to unrecoverable package startup
# and/or shutdown problems.
#
#-----------------------------------------------------------------------
# Description:  UPCC package run and halt script
#
# The existing package control script is modularized and divided into 
# 3 types of individual scripts:
#
# (1) The master control script, which uses the package configuration 
#     data retrieved via 'cmgetpkgenv' or a temporary environment
#     file to determine what actions to take and in what order.
#     Specifically, the "OPERATION_SEQUENCE[*]" attributes from
#     the package configuration data controls the module 
#     script execution order. 
#
# (2) The action scripts (aka module scripts), which are associated
#     with a Attribute Definition File (ADF) and perform module 
#     specific actions using the configured attributes defined in ADF.
#     For example:  ADF file service.1 is associated with module 
#                   script service.sh, which provides the operations to
#                   execute at package start-up, shutdown and validation
#                   time. 
# (3) The utility functions, which are used by the module scripts for 
#     rudimentary functions such as logging and tracing of the messages
#     to ensure an uniform package logging format. 
#
# Input :   The script accepts two parameters:
#
#     op_code :   The first argument is the the operation to be performed
#                 whose valid values are: start, stop, validate
#                 "start":    Calls each of the module scripts with 'start' 
#                             option. If error occurs, execute each previous 
#                             module script with stop parameter in reverse 
#                             order.
#                 "stop" :    Calls each of the module scripts with 'stop' 
#                             option
#                 "validate": Calls each of the module scripts with 'validate' 
#                             option. It requires a specification of the second 
#                             argument that specifies the package environment 
#                             filename. 
#     filename:   The second argument is the filename. It is optional and 
#                 needs to be specified only if op_code is set to "validate". 
#                 It specifies the name of the package environment file that 
#                 contains the package configuration attributes to be sourced 
#                 prior to validation. For "start" and "stop" operation, 
#                 package configuration is retrieved through the 'cmgetpkgenv' 
#                 command if there is no filename specifid.
#
# Output : Errors/Log messages. 
#          In case of "validate" the messages are printed to stdout.  
#          For "start" and "stop" cases the messages are directed to
#          $SG_SCRIPT_LOG_FILE. If the SCRIPT_LOG_FILE is not specified
#          the messages go to ${SGRUN}/log/${SG_PACKAGE_NAME}.log. 
#
# Returns:  0  When the execution of all the scripts was successful.
#           1  When there was error condition from the script and
#              package should not be restarted as a result of the error.
#           2  When there was error condition from the script, but the 
#              package can be can be restarted on an alternate node.
#
####################################################################

#
# Get parameters
#
if (( $# == 0 ))
then
   echo "ERROR: No parameters passed to command"
   exit 1 
fi 

#
# Export the global variable op_code  
#
typeset -x op_code="${1}"
#used in filesystem.sh
typeset -x master_node=
#used in volume_group.sh
typeset -x owner_node=


#
# Source in and export Serviceguard environment variables
# so they can be used in sub layer module scripts
#
set -a
. ${SGCONFFILE:=/etc/cmcluster.conf}
set +a

#
# PATH for HPUX and LINUX
#
PATH=$SGSBIN:/usr/bin:/usr/sbin:/etc:/bin:/sbin  
export PATH

#
# Utility functions file used by the module scripts
#
SG_UTILS=${SGCONF}/scripts/mscripts/utils.sh
export SG_UTILS


#
# Date format and hostname used by log messsages
#
SG_HOST_NAME=$(hostname)
if (( $? != 0 )) 
then 
    echo "ERROR: Error in getting the hostname" 
    sg_exit 1 $* 
fi

#
# variables SG_DATE_FORMAT & SG_LOG_PREFIX are defined
# in utils.sh and here. Any changes to these
# variables should be made in both the places.
#

SG_LOG_PREFIX="$(whoami)@$SG_HOST_NAME"
SG_DATE_FORMAT=${SG_DATE_FORMAT:-"+%b %e %H:%M:%S"}
export SG_LOG_PREFIX
export SG_HOST_NAME
export SG_DATE_FORMAT

#
# Platform-specific internal functions used by the module scripts
#
SG_INTERNAL_FUNCS_FILE=${SGCONF}/scripts/sg/internal.sh
export SG_INTERNAL_FUNCS_FILE

#
# Source in the utility functions used by the module scripts
#
if [[ -f ${SG_UTILS} ]]; then
    . ${SG_UTILS}
    if (( $? != 0 ))
    then
        echo "ERROR: Unable to source package utility functions file: ${SG_UTILS}"
        sg_exit 1 $*
    fi
else
    echo "ERROR: Unable to find package utility functions file: ${SG_UTILS}"
    sg_exit 1 $*
fi

#
# If SG_SCRIPT_LOG_FILE is already set in an enviornment, 
# and it is not the name of this script appended by .log, 
# proceed with that setting.  Otherwise, set the default package
# log to go to ${SGRUN}/log/<pkg_name>.log
#
if [[ -z $SG_SCRIPT_LOG_FILE ]]
then
      SG_SCRIPT_LOG_FILE=${SGRUN}/log/${SG_PACKAGE_NAME}.log
      export SG_SCRIPT_LOG_FILE
fi

#
# Get the environment for this package through utility function
# sg_source_pkg_env().
#
sg_source_pkg_env $*

#
# Validate default log directory exists
# If log directory does not exist create it.
#
if [[ ! -d ${SGRUN}/log ]]
then
    mkdir ${SGRUN}/log
    if (( $? != 0 ))
    then 
       sg_log 0 "ERROR: Creating directory ${SGRUN}/log"
       sg_exit 1 $*
    fi
    chmod 555 ${SGRUN}/log
fi

typeset pkg=$SG_PACKAGE_NAME
typeset state=

if [[ $SG_PACKAGE_TYPE = failover ]]
then
    state=$(cmviewcl -f line 2>/dev/null| awk -F = '/package:'$pkg'\|state=/ {print $2; exit}') 

    if [ -z "$state" ] ; then
        if (( $? == 0 ))
        then
            master_node=${SG_NODE_NAME[0]}
        else
            state=$(cmviewcl -f line 2>/dev/null | awk -F = '/package:'$pkg'\|state=/ {print $2; exit}')
            if [ -z "$state" ] ; then
                if (( $? == 0 ))
                then
                master_node=${SG_NODE_NAME[0]}
                else
                    sg_log 0 "cmviewcl -f line failed"
                    sg_exit 1 $*;      
                fi
            fi
        fi
    fi
     
    if [[ "$state" = "running" ]]
    then
        master_node=$(cmviewcl -f line 2>/dev/null | awk -F = '/package:'$pkg'\|owner=/ {print $2; exit}')
        if [ -z "$master_node" ] ; then 
            master_node=$(cmviewcl -f line 2>/dev/null | awk -F = '/package:'$pkg'\|owner=/ {print $2; exit}')
            if [ -z "$master_node" ] ; then
                sg_log 0 "cmviewcl -f line failed"
                sg_exit 1 $*;      
            fi
        fi
        owner_node=$master_node
    else
        master_node=${SG_NODE_NAME[0]}
    fi
fi

#
# Process the package validation/startup/shutdown sequence
#
# Set the size of an array of the operation sequence and call 
# the appropriate "utility" functions to execute the configured 
# module scripts.
# 
#

(( SG_OPERATION_SEQUENCE_SIZE = ${#SG_OPERATION_SEQUENCE[*]} ))

case "${op_code}" in 
    "start")    sg_execute_steps start
                ;;
    "stop")     sg_execute_steps stop
                ;;
    "validate") sg_execute_steps validate
                ;;
    "reconfigure") sg_execute_steps reconfigure
                   ;;
     *)         echo "INFO: Unknown operation - $op_code"
                # the $? is set to 1 when expression after let evaluates to 0
                let 0
                ;;
esac
sg_exit $? $* 
