####################################################################
# (C) Copyright 2006-2017 Hewlett Packard Enterprise Development LP.
# @(#) Serviceguard UPCC Package Configuration Script.
# @(#) Product Name                :  HP Serviceguard
# @(#) Product Version             :  A.12.10.00
# @(#) Patch Name                  :  
#
# *** Note: This file MUST NOT be edited. *****
#
# Any changes made to it will be overwritten when you upgrade to the
# next release of HP Serviceguard.
#
# Changing this file may lead to unrecoverable package startup
# and/or shutdown problems.
#
###################################################################

###########################
# Globals
typeset -i ERR_NO_SHARE=1
typeset -i ERR_NFSD_DOWN=2
############################
    
###########################
# Source utility functions.
###########################

if [[ -f ${SG_UTILS} ]]; then
    . ${SG_UTILS}
    if (( $? != 0 ))
    then
        echo "ERROR: Unable to source package utility functions file: ${SG_UTILS}"
        exit 1
    fi
else
    echo "ERROR: Unable to find package utility functions file: ${SG_UTILS}"
    exit 1
fi

###################################################################
#
# Get the environment for this package through utility function
# sg_source_pkg_env().
#
###################################################################

sg_source_pkg_env $*

function sg_check_nfsshare_ready
{
    typeset nfs_server
    typeset nfs_share
    typeset nfs_share_found
    typeset share
    typeset -i nfs_share_in_server=0
    typeset -i cmd_ret=0
    typeset sys_bin
    typeset -i func_ret=0
    
    nfs_server=$1
    shift
    nfs_share=$1

    cmd_out=`/usr/sbin/showmount -e ${nfs_server} 2>&1`
    conn_search_str="Timed out"
    cmd_ret=$?
    #Check connectivity to nfs server
    if [[ cmd_ret != 0 && $cmd_out =~ $conn_search_str ]]
    then
        return 4
    fi
    
    cmd_ret=0
    nfs_share_found=`showmount -e ${nfs_server} 2>/dev/null | \
        grep -vw ${nfs_server} 2>/dev/null | cut -f1 -d " " 2>/dev/null`
    cmd_ret=$?
    if [[ $cmd_ret = 0 ]]
    then
        for share in $nfs_share_found
        do
            if [[ $share = ${nfs_share} ]]
            then 
               nfs_share_in_server=1
            fi
        done
     fi        
    
    # Nfs share is not exported. 
    if (( nfs_share_in_server == 0 ))
    then
        (( func_ret = func_ret | $ERR_NO_SHARE ))
    fi    
        
    # share is exported. But check for nfsd. 
    # If nfsd is not running, mount fails. 

    # Set the path for rpcinfo command on RH and SLES 
    if [ -f /etc/redhat-release ]
    then
       sys_bin="/usr/sbin/"
    else
       sys_bin="/sbin"
    fi

    $sys_bin/rpcinfo -n 2049 -u $nfs_server nfs 2>/dev/null | grep -w "ready and waiting" >> /dev/null 
    cmd_ret=$?

    if [[ $cmd_ret != 0 ]] 
    then
        (( func_ret= func_ret | $ERR_NFSD_DOWN ))
    fi

    return $func_ret
}

###################################################################
#
# sg_validate_nfs_filesystems() is called inside sg_validate_filesystems().
# It verifies NFS file system specific attributes and conditions
#
###################################################################

function sg_validate_nfs_filesystems
{

    typeset -i ind=$1

    sg_log 5 "sg_validate_nfs_filesystems"

    # Check that NFS is specified for failover package   
    if [ ${SG_PACKAGE_TYPE} != "failover" ]  
    then  
        sg_log 0 "ERROR: Function sg_validate_nfs_filesystems"  
        sg_log 0 "       NFS filesystem is supported only for failover packages"  
        to_exit=1
    fi
		  
    # Verify NFS server is specified.
    if [[ -z ${SG_FS_SERVER[$ind]} ]]
    then
        sg_log 0 "ERROR: Function sg_validate_nfs_filesystems"
        sg_log 0 "       NFS server is not specified for NFS file system"
        to_exit=1
    else
        # Check whether the NFS share is exported on the NFS server
        sg_check_nfsshare_ready ${SG_FS_SERVER[$ind]} ${SG_FS_NAME[$ind]}
        ret=$?
        if (( ($ret & $ERR_NO_SHARE ) == $ERR_NO_SHARE )) 
        then
            sg_log 0 "ERROR: ${SG_FS_NAME[$ind]} is not exported in ${SG_FS_SERVER[$ind]}"
        fi
        if (( ($ret & $ERR_NFSD_DOWN) == $ERR_NFSD_DOWN ))
        then 
            sg_log 0 "ERROR: NFS filesystem cannot be mounted as nfsd is not running. "
            sg_log 0 "Restart the nfs service on ${SG_FS_SERVER[$ind]} to bring up nfsd"
        fi
        if (( $ret == 4 ))
        then
            sg_log 0 "ERROR: No connectivity to ${SG_FS_SERVER[$ind]}"
        fi
        (( to_exit = to_exit | $ret ))
    fi

    # Check whether fsck option is not specified for a NFS package
    if [[ -n ${SG_FS_FSCK_OPT[$ind]} ]]
    then
        sg_log 0 "WARNING: Function sg_validate_nfs_filesystems"
        sg_log 0 "         Filesystem - fsck option is not supported with NFS file system"
        sg_log 0 "         It will be ignored"
    fi
	
    return $to_exit
}

##########################################################################
#
# sg_validate_filesystems() is called while validating the package 
# configuration (through cmcheckconf/cmapplyconf)
#
##########################################################################

function sg_validate_filesystems
{
    typeset -i r=0
    typeset -i j=0
    typeset -i i=0
    # New variables to sort the mountpoints
    declare -a TMP_SG_FS_NAME
    declare -a TMP_SG_FS_SERVER
    declare -a TMP_SG_FS_DIRECTORY
    declare -a TMP_SG_FS_TYPE
    declare -a TMP_SG_FS_FSCK_OPT
    typeset -i num_fs
    typeset -i fs=0
    typeset -i cmserviced_pid
    typeset pstree_output=""
    typeset HOSTNAME="`echo ${SG_HOST_NAME%%.*} | tr '[:lower:]' '[:upper:]' | sed 's/\-/_/g'`"
    typeset array_ref="SG_${HOSTNAME}_PID[*]"
    typeset -i is_service_in_local_pkg=0
    typeset -i is_sg_service=0
    sg_log 5 "sg_validate_filesystems"

    # Refresh the mount command output
    cache_mount_output

    ## Sort the mountpoints - begin
    num_fs=${#SG_FS_NAME[*]}
    # assign mount points into temp array
    while (( $fs < $num_fs ))
    do
        TMP_SG_FS_NAME[$fs]=${SG_FS_NAME[$fs]}
        TMP_SG_FS_SERVER[$fs]=${SG_FS_SERVER[$fs]}
        TMP_SG_FS_TYPE[$fs]=${SG_FS_TYPE[$fs]}
        TMP_SG_FS_DIRECTORY[$fs]=${SG_FS_DIRECTORY[$fs]}
        TMP_SG_FS_FSCK_OPT[$fs]=${SG_FS_FSCK_OPT[$fs]}
        (( fs = fs + 1 ))
    done

    # call the sort_fs_directories function to sort the temp array
    sort_fs_directories

    fs=0
    while (( $fs < $num_fs ))
    do
        #validate if fs is already moounted
        check_and_validate_fs $fs
        if (( $? == 1 ))
        then
            sg_log 0 "ERROR: Function sg_validate_filesystems"
            sg_log 0 "       ${TMP_SG_FS_NAME[$fs]} is already mounted."
            sg_log 0 "       There is a mismatch of either the mount directory,"
            sg_log 0 "       mount type or both than what is specified in the package configuration."
            to_exit=1
            return $to_exit
        fi
        (( fs = fs + 1 ))
    done
    #
    # verify directory specified by SG_FS_NAME and SG_FS_DIRECTORY
    #
    while (( r < ${#TMP_SG_FS_NAME[*]} ))
    do
        #		
        # Validate fsck options for xfs filesystems		
        #		
        if [[ ${TMP_SG_FS_TYPE[$r]} = "xfs" ]]		
        then		
            echo ${TMP_SG_FS_FSCK_OPT[$r]} | grep -q "\-L"		
            ret=$?		
            if [[ $ret == 0 ]]		
                then		
                sg_log 0 "WARNING: Filesystem - Found that '-L' option is enabled for xfs ${TMP_SG_FS_NAME[$r]} filesystem."		
                sg_log 0 "         Using this option may cause corruption. See xfs_repair(8) for"
                sg_log 0 "         implications of using '-L' option."
            fi		
        fi

        if ! sg_check_if_storage_module_skip
        then
            #
            # Check existence of a logical volume
            #
            if [[ ! (((("${TMP_SG_FS_TYPE[$r]}" = "nfs")) || (("${SG_FS_TYPE[$r]}" = "vxfs")))) ]]
            then
                if [[ $(lvs --noheadings -o lv_name ${TMP_SG_FS_NAME[$r]}) = "" ]]
                then
                    sg_log 0 "ERROR: Function sg_validate_filesystems"
                    sg_log 0 "       Missing file ${TMP_SG_FS_NAME[$r]}"
                    sg_log 0 "       If you are using device-mapper for logical volumes," 
                    sg_log 0 "       make sure that the symbolic link is of the form"
                    sg_log 0 "       /dev/vg_dd0/lvol0 -> /dev/mapper/vg_dd0-lvol0"
                    to_exit=1
                fi
                #
                # Handle user mentioning volume group instead of a logical volume
                #    
                vgs --noheadings --nameprefixes -o vg_name ${TMP_SG_FS_NAME[$r]} 2>/dev/null | grep  -q 'LVM2_VG_NAME='
                if [[ $? = 0 ]]
                then
                    sg_log 0 "ERROR: Function sg_validate_filesystems"
                    sg_log 0 "      ${TMP_SG_FS_NAME[$r]} is a volume group."
                    sg_log 0 "      make sure that a valid logical volume is specified."
                    to_exit=1
                fi					 
            fi
        fi
		
        #
        # NFS's logical volume will be in remote NFS Server.
        # Check existence of a logical volume for non-NFS file system
        #
        if  [[ ${TMP_SG_FS_TYPE[$r]} = "nfs" ]]
        then
			sg_validate_nfs_filesystems $r
			to_exit=$?
        elif [[ -n ${TMP_SG_FS_SERVER[$r]} ]]
        then
			sg_log 0 "ERROR: Function sg_validate_filesystems"
			sg_log 0 '       "fs_server" is supported only with NFS file system'
			to_exit=1
        fi
        
        #
        # Check existence of a mount directory 
        # only if it is not a nested mountpoint
        #
        (( ${mnt_order[$r]} == 1 )) &&  if [[ ! -d ${TMP_SG_FS_DIRECTORY[$r]} ]]
        then
            sg_log 0 "ERROR: Function sg_validate_filesystems"
            sg_log 0 "       Missing directory ${TMP_SG_FS_DIRECTORY[$r]}"
            to_exit=1
        fi

        if ! sg_check_if_storage_module_skip
        then
            #
            # For GFS validate :
            #
            # SG_FS_UMOUNT_RETRY_COUNT=1
            # SG_FS_MOUNT_RETRY_COUNT=0
            # SG_CONCURRENT_FSCK_OPERATIONS=1
            #

            if [[ "${TMP_SG_FS_TYPE[$r]}" == "gfs" ]]
            then

                if (( SG_FS_UMOUNT_RETRY_COUNT != 1 ))
                then
                    sg_log 0 "ERROR: Function sg_validate_filesystems"
                    sg_log 0 "       For the GFS file type the value of SG_FS_UMOUNT_COUNT needs to be 1"
                    to_exit=1
                fi

                if (( SG_FS_MOUNT_RETRY_COUNT != 0 ))
                then
                    sg_log 0 "ERROR: Function sg_validate_filesystems"
                    sg_log 0 "       For the GFS file type the value of SG_FS_MOUNT_RETRY_COUNT needs to be 0"
                    to_exit=1
                fi

                if (( SG_CONCURRENT_FSCK_OPERATIONS != 1 ))
                then
                    sg_log 0 "ERROR: Function sg_validate_filesystems"
                    sg_log 0 "       For the GFS file type the value of SG_CONCURRENT_FSCK_OPERATIONS needs to be 1"
                    to_exit=1

                fi
            fi
        fi

        (( r = r + 1 ))
    done
    cmserviced_pid=`pgrep cmserviced` 2>/dev/null
    is_cmserviced_pid=$?
    pstree_output=`pstree -Alp $cmserviced_pid 2>/dev/null`
    is_cmserviced_pid=$is_cmserviced_pid || $?
    if [[ $is_cmserviced_pid == 0 ]]
    then
        fs=0
        for curr_fs in ${SG_FS_NAME[@]}
        do
	    echo $curr_fs | grep -q "\/mapper\/" 2>/dev/null
            if [[ $? != 0 ]]
            then 
                curr_fs_format=`echo $curr_fs |  sed 's/\/dev\///' | \
                               sed 's/-/--/' | sed 's/\//-/'`
                curr_fs="/dev/mapper/"$curr_fs_format
            fi
            TMP_CURR_SG_FS_NAME[$fs]=`echo $curr_fs`
            (( fs = fs + 1 ))
        done

        for prev_fs in ${PREV_SG_FS_NAME[@]}
        do
            echo $prev_fs | grep -q "\/mapper\/" 2>/dev/null
            if [[ $? != 0 ]]
            then 
                prev_fs_format=`echo $prev_fs |  sed 's/\/dev\///' | \
                               sed 's/-/--/' | sed 's/\//-/'`
                prev_fs="/dev/mapper/"$prev_fs_format
            fi
            echo ${TMP_CURR_SG_FS_NAME[@]}|grep -q "$prev_fs"
            if [[ $? != 0 ]]
            then
                # Do fuser and check if some process is using the fs
                fs_proc=`fuser -m $prev_fs 2>/dev/null`
                if [[ $fs_proc != "" ]]
                then
                    for pid in $fs_proc
                    do
                        is_sg_service=0 
                        for p_pid in $pstree_output
                        do
                            `echo $p_pid | grep -q $pid`
                            if [[ $? == 0 ]]
                            then
                                # The pid of the process accessing the fs is one spawned by
                                # cmserviced. Find the service command from pstree output
                                # The pstree output can be as below, so we need to extract all the
                                # pids for comparison. This sort of a tree output seen, when the
                                # service_cmd spawns multiple child and one of the chile is using 
                                # the fs.
                                # In this example array_ref will be pointing to 15847
                                # and fs_proc will have 15853 & 21484
                                # cmserviced(10337)-+-bash(15847)---bash(15850)---sh(15851)---sh(15853)---sleep(21484)
                                # |-cmfileassistd(10340)
                                # |-cmlogd(10339)
                                # `-cmnetd(10341)

                                is_sg_service=1
                                parent_pids=`echo $p_pid | sed '/\(^[^(]*(\|)[^(]*(\|)[^(]*\)/s// /g;s/ //'`

                                # as per the above example parent_pids will have "10337 15847 15850 15851 15853 21484"
                                # srv_pid will have 15853.
                                # Check if this pid is present in the sg_<node>_pid array

                                i=0
                                is_service_in_local_pkg=0 
                                for srv_pid in ${!array_ref} 
                                do
                                    for parent_pid in $parent_pids
                                    do
                                        if [[ $srv_pid == $parent_pid ]]
                                        then
                                            sg_log 0 "ERROR: The service ${SG_SERVICE_NAME[$i]} is using the filesystem $prev_fs"
                                            sg_log 0 "Hence, this filesystem cannot be removed."
                                            sg_log 0 "Remove the service ${SG_SERVICE_NAME[$i]}:$pid"
                                            sg_log 0 "in order to remove the filesystem."
                                            to_exit=1
                                            is_service_in_local_pkg=1
                                        fi
                                    done
                                    (( i = i + 1 ))
                                done
                                # If the pid is not found as part of sg_<node>_pid array, there are two cases :
                                # 1. The service accessing the fs is being removed from the curent package or
                                # 2. The service may be present or removed from another package.

                                if [[ $is_service_in_local_pkg == 0 ]]
                                then
                                    #TO-DO
                                    :
                                fi
                            fi
                        done
                        if [[ $is_sg_service != 1 ]]
                        then
                            serv_cmd=`ps -ef | awk -v x=$pid '$2 == x {print $0}'` 
                            serv_cmd=`echo $serv_cmd | cut -d " " -f 8- `
                            sg_log 0 "WARNING: Filesystem - The filesystem $prev_fs used by the process"
                            sg_log 0 "$serv_cmd:$pid is being removed."
                            sg_log 0 "Stop the process $serv_cmd:$pid before removing the filesystem." 
                        fi
                    done
                fi
            fi
        done
    fi
}

function is_mounted_wrongly
{
   typeset mnt_dev=$1
   typeset mnt_dir=$2
   typeset fs_type=$3

   if [[ "$fs_type" != "gfs" && "$fs_type" != "nfs" && "$fs_type" != "vxfs" ]]
   then
       echo $mnt_dev | grep -q "\/dev\/mapper\/"
       ret=$?
       if [[ $ret != 0 ]]
       then
           mnt_dev=${mnt_dev/\/dev\/}
           mnt_dev=`echo $mnt_dev | sed 's/-/--/g'`
           mnt_dev=${mnt_dev/\//-}
       fi
   fi
   $(echo ${MOUNT_CMD_OUTPUT[@]} | grep -q "$mnt_dev on ")
   if [[ $? == 0 ]]
   then
       sg_log 3 "$mnt_dev is incorrectly mounted with either mount directory or filesystem type"
       return 1
   else
       return 0
   fi
}

function check_and_validate_fs
{
    typeset -i ind=$1
    typeset -i a_num_fs
    typeset -i a_fs=0
    
    sg_log 5 "check_and_validate_fs"
    is_already_mounted ${TMP_SG_FS_NAME[$ind]} ${TMP_SG_FS_DIRECTORY[$ind]} ${TMP_SG_FS_TYPE[$ind]} 
    if [[ $? == 0 ]]
    then 
        is_mounted_wrongly ${TMP_SG_FS_NAME[$ind]} ${TMP_SG_FS_DIRECTORY[$ind]} ${TMP_SG_FS_TYPE[$ind]}
        if [[ $? == 1 ]]
        then
            return 1;
        else 
            return 0
        fi
    fi
    return 0;
}

####################################################
#
# sort_fs_directories() is used to sort all the
# mount points for a package. This is used to order
# mounts when nested mount points are configured such
# that parent mount is done before trying to mount
# child
#
##################################################### 

function sort_fs_directories
{
    typeset -i r=0
    typeset -i i=0
    typeset -i a=0
    declare -a indexes
    declare -a num_of_comp
    typeset Y
    typeset X
    typeset dirnm
    typeset temp
    typeset temp1
    typeset -i j

    while (( $a < ${#TMP_SG_FS_DIRECTORY[*]} ))
    do
        (( indexes[a] = $a ))
        (( mnt_order[a] = 1 ))

        #Make sure all the mnt points end with a / for uniformity
        Y=${TMP_SG_FS_DIRECTORY[a]}
        if [[ ${Y%/} = $Y ]]
        then
            Y=$Y/
        fi
        dirnm=$Y

        # Calculate number of components in the moint point path
        (( num_of_comp[a] = $(echo $dirnm | awk -F "/" '{print NF}') ))
        (( a = a + 1 ))
    done
      
    sg_log 3 "Number of components: ${num_of_comp[*]}"

    ((i=0))
    ((temp=0))
    ((temp1=0))
      
    #sort num_of_comp[*] and save the indexes as we have to
    #with FS and FS_options arrays

    while ((i<${#num_of_comp[*]}))
    do
        ((j=i+1))
        while ((j < ${#num_of_comp[*]}))
        do
            if (( ${num_of_comp[i]} > ${num_of_comp[j]} ))
            then
                temp=${num_of_comp[i]}
                num_of_comp[i]=${num_of_comp[j]}
                num_of_comp[j]=$temp

                temp1=${indexes[i]}
                indexes[i]=${indexes[j]}
                indexes[j]=$temp1
            fi
            ((j=j+1))
        done
        ((i=i+1))
    done
    sg_log 3 "sorted indexes: ${indexes[*]}"

    #Loop through all the elements starting from the last
    #element to the first to find out if subcomponents of
    #it need to be mounted before this can be mounted.

    (( max_mnt_order = 1 ))
    ((i=${#indexes[*]} - 1))
    while ((i>-1))
    do
        sg_log 3 "processing $i ${TMP_SG_FS_DIRECTORY[${indexes[i]}]} ${mnt_order[i]}"
        X=${TMP_SG_FS_DIRECTORY[${indexes[i]}]}
        ((j=0))
        while ((j<i))
        do
            Y=${TMP_SG_FS_DIRECTORY[${indexes[j]}]}

            #See if $Y is a substring of $X
            if [[ ${X#$Y} = $X ]]
            then
                ((j=j+1))
            else
                #If $Y is a substring  of $X it means $Y is parent
                #of $X. So, $X needs to be mounted after $Y. So, increment
                #corresponding mnt_order[] element by 1.

                ((mnt_order[${indexes[i]}]=${mnt_order[${indexes[i]}]}+1))
                if (( ${mnt_order[${indexes[i]}]} > $max_mnt_order ))
                then
                    (( max_mnt_order = max_mnt_order + 1 ))
                fi
                ((j=j+1))
            fi
        done
        ((i=i-1))
    done
    sg_log 4 "Mount order: ${mnt_order[*]}"
}


#######################################################################
#
# sg_check_and_mount() is called while starting a package (cmrunpkg)
#
# For each {file system/device} pair, fsck the file system
# and mount it.
#
# If the underlying file system is Red Hat GFS, mount the file system
# if it is not already mounted; fsck operations are not performed for
# GFS.
#
# If the mount point is busy and if FS_MOUNT_RETRY_COUNT = 0,
# mounting of the file system will fail and the control script
# will exit with an error.
# 
#######################################################################

function sg_check_and_mount
{
    typeset pids_list
    typeset fs_list
    typeset failed_to_repair
    typeset -i NOT_READY=1
    typeset -i R=0
    typeset -i F=0
    typeset -i j=0
    typeset -i retval=0
    typeset -i exit_value=0
    declare -a mnt_order
    typeset -i total_mnts
    typeset -i cur_ord
    typeset -i mnts_processed
    typeset -i mnts_identified
    typeset RHEL5VERSION="5\.[[:digit:]]"
    typeset regexp_I
	
    sg_log 5 "sg_check_and_mount"

    while (( $NOT_READY != 0 ))
    do
       NOT_READY=0
       for I in ${TMP_SG_FS_NAME[@]}
       do
         if [[ $(mount | awk '$1 == "'$I'"') = "" ]]
         then
           case $I in
              */dev/vx/dsk*)
                  check_vxvm_vol_available $I
                  if(( $? != 0 ))
                  then
                      NOT_READY=1
                  fi
                  ;;
              *)
           esac
         fi
       done
    done

    # Verify that there is at least one file system to check and what type.
    if (( ${#TMP_SG_FS_NAME[*]} > 0 ))
    then
        sg_log 0 "Checking filesystems:"
        echo ${TMP_SG_FS_NAME[@]} | tr ' ' '\012' | sed -e 's/^/   /'
        
        # Perform parallel fsck's for better performance.
        # Limit the number of concurrent fsck's to SG_CONCURRENT_FSCK_OPERATIONS
        
        R=0
        while (( R < ${#TMP_SG_FS_NAME[*]} ))
        do
            j=0
            failed_to_repair[$j]=0
            while (( j < SG_CONCURRENT_FSCK_OPERATIONS  && R < ${#TMP_SG_FS_NAME[*]} ))
            do
                failed_to_repair[$j]=0
                (
                check_if_mounted ${TMP_SG_FS_NAME[$R]} ${TMP_SG_FS_TYPE[$R]} ${TMP_SG_FS_DIRECTORY[$R]}
                do_fsck=$?
                if [[ $do_fsck == 2 ]]
                then
                    sg_log 0 "ERROR: Function sg_check_and_mount"
                    sg_log 0 "       ${TMP_SG_FS_NAME[$R]} is already mounted."
                    sg_log 0 "       There is a mismatch of either the mount directory," 
                    sg_log 0 "       mount type or both than what is specified in the package configuration."
                    exit 2
                fi
                
                case ${TMP_SG_FS_TYPE[$R]} in
                    ext2|ext3)
                        if [[ $do_fsck == 0 ]]
                        then
                            e2fsck ${TMP_SG_FS_FSCK_OPT[$R]} -y ${TMP_SG_FS_NAME[$R]}
                        fi
                        ;;
                    ext4)
                        if [[ $do_fsck == 0 ]] 
                        then
                            if ([ -f "/etc/redhat-release" ] && \
                                (cat /etc/redhat-release | grep -q $RHEL5VERSION)) then
                                e4fsck ${TMP_SG_FS_FSCK_OPT[$R]} -y ${TMP_SG_FS_NAME[$R]}
                            else
                                e2fsck ${TMP_SG_FS_FSCK_OPT[$R]} -y ${TMP_SG_FS_NAME[$R]}
                            fi
                        fi
                        ;;
                    reiserfs)
                        if [[ $do_fsck == 0 ]]
                        then
                            fsck -a -t ${TMP_SG_FS_TYPE[$R]} ${TMP_SG_FS_FSCK_OPT[$R]} ${TMP_SG_FS_NAME[$R]}
                        fi
                        ;;
                    vxfs)
                        if [[ $do_fsck == 0 ]]
                        then
                            fsck -t ${TMP_SG_FS_TYPE[$R]} ${TMP_SG_FS_FSCK_OPT[$R]} -P -y ${TMP_SG_FS_NAME[$R]}
                        fi
                        ;; 
                    nfs|gfs) : # do nothing for nfs or gfs
                        ;;
                    xfs|btrfs)
                        (
                        if [[ $do_fsck == 0 ]]
                        then
                            mount -t ${TMP_SG_FS_TYPE[$R]} ${TMP_SG_FS_MOUNT_OPT[$R]} ${TMP_SG_FS_NAME[$R]} ${TMP_SG_FS_DIRECTORY[$R]}
                            if [[ $? != 0 ]]
                            then
                                if [[ ${TMP_SG_FS_TYPE[$R]} == "xfs" ]]
                                then
                                    /sbin/xfs_repair ${TMP_SG_FS_FSCK_OPT[$R]} ${TMP_SG_FS_NAME[$R]}
                                    ret=$?
                                    if [[ $ret > 0 ]]
                                    then
                                        failed_to_repair[$j]=1
                                        sg_log 0 "WARNING: xfs_repair failed for ${TMP_SG_FS_NAME[$R]}."
                                    fi
                                else 
                                    /sbin/btrfsck ${TMP_SG_FS_FSCK_OPT[$R]} ${TMP_SG_FS_NAME[$R]}
                                    ret=$?
                                    if [[ $ret > 0 ]]
                                    then
                                        failed_to_repair[$j]=1
                                        sg_log 0 "WARNING: btrfsck failed for ${TMP_SG_FS_NAME[$R]}."
                                    fi
                                fi
                            else
                                umount -f ${TMP_SG_FS_UMOUNT_OPT[$R]} ${TMP_SG_FS_NAME[$R]}
                                if [[ $? != 0 ]]
                                then
                                    sg_log 0 "Failed to umount ${TMP_SG_FS_MOUNT_OPT[$R]}."
                                    failed_to_repair[$j]=1
                                fi
                            fi
                        fi
                        )
                        ;;
                    *)
                        if [[ $do_fsck == 0 ]]
                        then
                            fsck -p -T -t ${TMP_SG_FS_TYPE[$R]} ${TMP_SG_FS_NAME[$R]}
                        fi
                        ;;
                esac ) &
                if [[ ${TMP_SG_FS_TYPE[$R]} != "nfs" ]]
                then 
                    # save the process id and fs for monitoring the status
                    pids_list[$j]="$!"
                    fs_list[$j]=${TMP_SG_FS_NAME[$R]}
                    (( j = j + 1 ))
                fi
                (( R = R + 1 ))
            done
            # wait for background fsck's to finish
            while (( j > 0 ))
            do
                pid=${pids_list[$j-1]}
                I=${fs_list[$j-1]}
                wait $pid
                # on linux fsck will return a 1 if file system errors
                # were corrected. This means that the file system was
                # dirty but is now clean so we should be able to
                # continue. Any other errors we should abort as normal.
                # xfs_repair will always return 0; details have to be
                # checked in package log files
                if (( $? > 1 || ${failed_to_repair[$j-1]} != 0 ))
                then
                    if sg_check_if_it_is_reconfiguration
                    then
                        sg_log 999 "ERROR: Failed to fsck $I."
                        sg_set_reconfig_failure
                    else
                        sg_log 0 "ERROR: Function sg_check_and_mount"
                        sg_log 0 "ERROR: Failed to fsck $I."
                    fi
                    exit_value=1
                else
                    if sg_check_if_it_is_reconfiguration
                    then
                        sg_log 999 "SUCCESS: fsck succeded on $I."
                    fi
                fi
                (( j = j - 1 ))
            done
        done
    fi

    # Check exit value (set if any proceeding fsck calls failed)
    if (( exit_value == 1 ))
    then
        if sg_check_if_it_is_reconfiguration
        then
            to_exit=1
        else
            sg_log 0 "ERROR: Function sg_check_and_mount"
            sg_log 0 "ERROR: Preceeding fsck call failed"
            to_exit=1
            return
        fi
    fi
	
    sort_fs_directories
    #Total number of mounts to do
    (( total_mnts = ${#mnt_order[*]} ))

    #Current order of mount being processed
    (( cur_order = 1 ))
    #Used to keep track how many mounts are
    # completed
    (( mnts_processed = 0 ))

    #Number of mounts being done in one iteration
    (( mnts_identified = 0 ))

    #Variable to loop through mount point array
    typeset -i F=0 
    typeset -i L=${#TMP_SG_FS_NAME[*]}
    typeset volume_list

    while (( mnts_processed < total_mnts ))
    do
        mnts_identified=0
      
        #If we have gone through all the
        #list then we can process next level of mounts

        if (( $F == $total_mnts ))
        then
            (( cur_order = cur_order + 1 ))
            (( F = 0 ))
        fi

        while (( mnts_identified < \
            $TMP_SG_CONCURRENT_MOUNT_AND_UMOUNT_OPERATIONS && F < total_mnts ))
        do
            if (( ${mnt_order[$F]} == $cur_order ))
            then
                I=${TMP_SG_FS_NAME[$F]}
                K=${TMP_SG_FS_DIRECTORY[$F]}
                if sg_check_if_reconfig_has_failed
                then
                    sg_log 999 "WARNING: Not attempting to mount $I on $K"
                else
                    regexp_I=$I

                    # NFS mount command is specified with NFS Server name. If NFS Server is
                    # IPv6 address, it has to be enclosed in [] brackets.  However
                    # the grep command used to find whether it is already mounted
                    # grep requires the '[' ']' characters to be escaped so that they
                    # are not considered as special characters

                    if [[ ${TMP_SG_FS_SERVER[$F]} != "" ]]
                    then
                        echo ${TMP_SG_FS_SERVER[$F]} | grep -q ":"
                        ret=$?
                        if [[ $ret = 0 ]]
                        then
                            regexp_I="\["${TMP_SG_FS_SERVER[$F]}"\]":$I
                            I="["${TMP_SG_FS_SERVER[$F]}"]":$I
                        else
                            I=${TMP_SG_FS_SERVER[$F]}:$I
                            regexp_I=$I
                        fi
                    fi

                    if [[ ${TMP_SG_FS_TYPE[$F]} != "nfs" ]]
                    then
                        sg_log 0 "Logical Volume is $regexp_I "
                    else
                        sg_log 0 "NFS Share is $regexp_I "
                    fi
                        
                    sg_log 0 "Mounting $regexp_I with option ${TMP_SG_FS_MOUNT_OPT[$F]} on ${TMP_SG_FS_DIRECTORY[$F]}"
           
                    is_already_mounted $regexp_I ${TMP_SG_FS_DIRECTORY[$F]} ${TMP_SG_FS_TYPE[$F]} 

                    if (( $? == 0 ))
                    then
                        sg_log 0 "Mounting $regexp_I at ${TMP_SG_FS_DIRECTORY[$F]}"
                        # Perform parallel file system mounts for better performance.
                        # Limit the number of parallel mounts to
                        # SG_CONCURRENT_MOUNT_AND_UMOUNT_OPERATIONS

                        # if there is permission to kill the user, we can
                        # run fuser to kill the user, on the mount point.
                        # This would freeup the mount point, if it is busy

            
                        if (( SG_FS_MOUNT_RETRY_COUNT > 0 ))
                        then (
                            mount -t ${TMP_SG_FS_TYPE[$F]} ${TMP_SG_FS_MOUNT_OPT[$F]} $regexp_I ${TMP_SG_FS_DIRECTORY[$F]}
                            if (( $? != 0 ))
                            then
                                freeup_busy_mountpoint_and_mount_fs $regexp_I ${TMP_SG_FS_DIRECTORY[$F]} \
                                -t ${TMP_SG_FS_TYPE[$F]} ${TMP_SG_FS_MOUNT_OPT[$F]}
                                (( retval=$? ))
                                return $retval
                            fi
                        ) &
                        else (
                            mount -t ${TMP_SG_FS_TYPE[$F]} ${TMP_SG_FS_MOUNT_OPT[$F]} $regexp_I ${TMP_SG_FS_DIRECTORY[$F]}
                        ) &
                        fi
                        # save the process id and volume name for monitoring status
                        pids_list[$mnts_identified]="$!"
                        volume_list[$mnts_identified]=$I
                        fs_dir_list[$mnts_identified]=$K
                        (( mnts_identified = mnts_identified + 1 ))
                    else
                        #
                        # fs is already mounted
                        #
                        pids_list[$mnts_identified]=""
                        volume_list[$mnts_identified]=$I
                        fs_dir_list[$mnts_identified]=$K
                        sg_log 0 "WARNING: File system \"${TMP_SG_FS_DIRECTORY[$F]}\" was already mounted."
                    fi
                fi
                (( mnts_processed = mnts_processed + 1 ))
            fi #new
            (( F = F + 1 ))
        done
      
         # wait for background mounts to finish
        while (( mnts_identified > 0 )) && (( exit_value == 0 ))
        do
            pid=${pids_list[$mnts_identified-1]}
            I=${volume_list[$mnts_identified-1]}
            K=${fs_dir_list[$mnts_identified-1]}
            wait $pid
            if (( $? != 0 ))
            then
                if sg_check_if_it_is_reconfiguration
                then
                    sg_log 999 "ERROR: Failed to mount $I on $K"
                    sg_set_reconfig_failure
                else
                    sg_log 0 "ERROR: Function sg_check_and_mount"
                    sg_log 0 "ERROR: Failed to mount $I"
                fi
                exit_value=1
            else
                if sg_check_if_it_is_reconfiguration
                then
                    sg_log 999 "SUCCESS: Successfully mounted $I on $K"
                fi
            fi
            (( mnts_identified = mnts_identified - 1 ))
        done
      
      # Check exit value (set if any preceeding mount calls failed)
        if (( exit_value == 1 ))
        then
            if sg_check_if_reconfig_has_failed
            then
                to_exit=1
                continue
            else
                to_exit=1
                return
            fi
        fi

    done
    return 0
}
#######################################################################
#
# This function is wrapper to is_already_mounted function. It also 
# checks if the fs is mounted with the same directory i.e specified in 
# package ascii files. 
# 
#######################################################################

function check_if_mounted
{
    typeset mnt_dev=$1
    typeset fs_type=$2
    typeset fs_dir=$3
    typeset -i retval=0
    case $fs_type in
         nfs|gfs) : # do nothing for nfs or gfs or xfs or btrfs
         ;;
         *)
         is_already_mounted $mnt_dev $fs_dir $fs_type
         if [[ $? == 0 ]]
         then
             is_mounted_wrongly $mnt_dev $fs_dir $fs_type
             if [[ $? == 1 ]]
             then
                 retval=2
             else
                 retval=0
             fi
         else 
             retval=1
         fi
         ;;
    esac
    return $retval
}

function is_already_mounted
{
   typeset mnt_dev=$1
   typeset mnt_dir=$2
   typeset fs_type=$3

   if [[ "$fs_type" != "gfs" && "$fs_type" != "nfs" && "$fs_type" != "vxfs" ]]
   then
       echo $mnt_dev | grep -q "\/dev\/mapper\/"
       ret=$?
       if [[ $ret != 0 ]]
       then
           mnt_dev=${mnt_dev/\/dev\/}
           mnt_dev=`echo $mnt_dev | sed 's/-/--/g'`
           mnt_dev=${mnt_dev/\//-}
       fi
   fi

   $(echo ${MOUNT_CMD_OUTPUT[@]} | grep -q "$mnt_dev on $mnt_dir type $fs_type") 
   if [[ $? == 0 ]]
   then
       return 1
   else
       return 0
   fi
}

######################################################################
#
#This function is used to kill the user to freeup a mountpoint
#that could be busy and then do the mount operation.
#freeup_busy_mountpoint_and_mount_fs(x, y, z) 
#	x  =  Logical volume group to be mounted.
#	y  =  File System where the logical volume is to be mounted.
#       z  =  Mount Options to be used for mount operation
#
######################################################################

function freeup_busy_mountpoint_and_mount_fs
{

    typeset vol_to_mount
    typeset mount_pt
    typeset fs_mount_opt
    typeset -i UM_COUNT=0
    typeset -i retval=1

    sg_log 0 "freeup_busy_mountpoint_and_mount_fs"
 
    vol_to_mount=$1
    mount_pt=$2
    shift 2
    fs_mount_opt=$*
   
    sg_log 0 "WARNING: Running fuser on ${mount_pt} to remove anyone using the busy mount point directly."

   # The control script exits, if the mount failed after 
   # retrying SG_FS_MOUNT_RETRY_COUNT times. 

    while (( UM_COUNT < SG_FS_MOUNT_RETRY_COUNT && retval != 0 ))
    do
        (( UM_COUNT = UM_COUNT + 1 ))
        fuser -ku ${mount_pt}
        if (( UM_COUNT == $SG_FS_MOUNT_RETRY_COUNT ))
        then
            mount ${fs_mount_opt} ${vol_to_mount} ${mount_pt}
            retval = $? 
            if (( retval != 0 ))
            then
                sg_log 0 "ERROR: Function freeup_busy_mountpoint_and_mount_fs"
                sg_log 0 "ERROR: Failed to mount ${vol_to_mount} to ${mount_pt}"
            break
            fi
        else
            mount ${fs_mount_opt} ${vol_to_mount} ${mount_pt}
            retval = $? 
            sleep 1
        fi
    done
    return $retval
}

#######################################################################
#
# check_vxvm_vol_available() is called by sg_check_and_mount()
# during package startup.
#
#######################################################################

function check_vxvm_vol_available
{
    typeset volpath
    typeset VOL
    typeset TMP
    typeset DG
    typeset vol_kstate

    sg_log 5 "check_vxvm_vol_available"

    volpath=$1
    VOL=${volpath##*/}
    TMP=${volpath%/*}
    DG=${TMP##*/}
    vol_kstate=$(vxprint -g $DG -F %kstate $VOL)
    TMP=$?
    if (( TMP != 0 ))
    then
        sg_log 0 "ERROR: Function check_vxvm_vol_available"
        sg_log 0 "ERROR: vxprint -g $DG -F %kstate $VOL"
        sg_log 0 "ERROR: Failed to get KSTATE for \"${volpath}\" See vxintro(1M) for EXIT CODE of $TMP."
        return 0
    fi

    if [[  $vol_kstate = "ENABLED" ]]
    then
        return 0
    fi
    sg_log 0 "KSTATE for \"${volpath}\" is ${vol_kstate}."
    return 1
}

##########################################################################
#
# sg_umount_fs() is called while stopping a package (cmhaltpkg)
# or during rollback operation to recover from the package startup.
#
##########################################################################

function sg_umount_fs
{
    typeset -i j=0
    typeset -i L
    typeset -i UM_CNT=${SG_FS_UMOUNT_RETRY_COUNT:-1}
    typeset -i ret
    typeset -i exit_value=0
    declare -a mnt_order
    typeset -i max_mnt_order=1
    typeset -i i=0
    typeset I
    typeset FSS
    typeset -i nfs_server_ipv6=0
    typeset VXUMOUNT="/usr/lib/fs/vxfs/vxumount"

    sg_log 5 "sg_umount_fs"

    sort_fs_directories
    sg_log 3 "max mnt order: $max_mnt_order"

    L=${#TMP_SG_FS_NAME[*]}
    # Perform parallel file system umounts for better performance.
    # Limit the number of parallel umounts to
    #    SG_CONCURRENT_MOUNT_AND_UMOUNT_OPERATIONS
 
    #Current order of mount being processed
    (( cur_order = max_mnt_order ))

    #Used to keep track how many mounts are completed
    (( mnts_processed = 0 ))

    #Total number of mounts to do
    (( total_mnts = ${#mnt_order[*]} ))

    #Number of mounts being done in one iteration
    (( mnts_identified = 0 ))

    #Variable to loop through mount point array
    (( i = total_mnts ))

    typeset mounttab=$(mount) 
    typeset pids_list
    typeset volume_list
    typeset fs_dir_list

    while (( mnts_processed < total_mnts )) 
    do
        (( mnts_identified = 0 ))

        #Check if we have gone through all
        #the elements, if so process to unmount
        # previous level mounts
        if (( $i == 0 ))
        then
            (( cur_order = cur_order - 1 ))
            (( i = $total_mnts ))
        fi

        while (( mnts_identified < \
             $TMP_SG_CONCURRENT_MOUNT_AND_UMOUNT_OPERATIONS && i > 0 ))
        do
            (( i = i - 1 ))
            if (( ${mnt_order[$i]} == $cur_order ))
            then
                I=${TMP_SG_FS_NAME[$i]}
                FSS=${TMP_SG_FS_SERVER[$i]}
                nfs_server_ipv6=0;

                if [[ $FSS != "" ]]
                then
                    # If IPv6 address it has be enclosed in []
                    echo $FSS | grep -q ":"
                    ret=$?
                    if [[ $ret = 0 ]]
                    then
                        nfs_server_ipv6=1
                        I="["$FSS"]":$I
                    else
                        I=$FSS:$I
                    fi

                    sg_check_nfsshare_ready ${TMP_SG_FS_SERVER[$i]} ${TMP_SG_FS_NAME[$i]}
                    ret=$?

                    # NFS Server is not accessible. Do forcible unmount
                    if [[ $ret != 0 ]]
                    then
                        sg_log 0 "Unmounting file system $I using the -f option"
                        (
                        fuser -ku ${TMP_SG_FS_DIRECTORY[$i]}
                        umount -f ${TMP_SG_FS_UMOUNT_OPT[$i]} ${TMP_SG_FS_DIRECTORY[$i]};
                        ret=$?
                        io_timeout=`cmviewcl -v -f line -s config 2> /dev/null | \
                                    grep configured_io_timeout_extension | \
                                    awk -F"=" '{ print $NF }'`
                        if [ -z "$io_timeout" ]
                        then
                            sg_log 0 "cmviewcl FAILED, trying again"
                            io_timeout=`cmviewcl -v -f line -s config 2> /dev/null | \
                                        grep configured_io_timeout_extension | \
                                        awk -F"=" '{ print $NF }'`

                            if [ -z "$io_timeout" ]
                            then
                                sg_log 0 "cmviewcl FAILED again, Using 0 as default \
                                          value for configured_io_timeout_extension"
                                io_timeout=0
                            fi
                        fi

                        # CONFIGURED_IO_TIMEOUT_EXTENSTION parameter defined in Cluster ascii file
                        # has unit in micro-seconds. Convert it to seconds before use.
                        let io_timeout=$io_timeout/1000000
                        sg_log 0 "Waiting for $io_timeout seconds to complete any pending IO to $I"
                        sleep $io_timeout
                        return $ret
                        ) &

                        pids_list[$mnts_identified]="$!"
                        volume_list[$mnts_identified]=$I
                        fs_dir_list[$mnts_identified]=$FSS
                        (( mnts_identified = mnts_identified + 1 ))
                        
                        # Here NFS server is not available and forceful umount has been done.
                        # set ret to 1, so that it should not try to umount again.
                        ret=1;
                    elif [[ $nfs_server_ipv6 = 1 ]]
                    then
                        tmp_mnt_loc=`echo $I | sed -e 's/\[/\\\[/'| sed -e 's/\]/\\\]/`
                        echo $mounttab | grep -e $tmp_mnt_loc" " > /dev/null 2>&1
                        ret=$?
                    else
                        echo $mounttab | grep -e $I" " > /dev/null 2>&1
                        ret=$?
                    fi
                else
                    # It is not a NFS share. function is_already_mounted returns 0 if not mounted else returns 1.
                    # Toggle the return status as we needed the status other way.
                    is_already_mounted $I ${TMP_SG_FS_DIRECTORY[$i]} ${TMP_SG_FS_TYPE[$i]} 
                    ret=$?
                    let ret=$ret^1
                fi

                if (( $ret == 0 ))
                then
                #
                # Don't unmount gfs filesystem 
                #
                    if [[ "${TMP_SG_FS_TYPE[$i]}" == "gfs" ]]
                    then 
                        return 0
                    fi
                    K=${TMP_SG_FS_DIRECTORY[$i]}
                    I=${TMP_SG_FS_NAME[$i]}
                    mount | grep -e " "${K%/}" " > /dev/null 2>&1
                    if (( $? == 0 ))
                    then
                        sg_log 0 "Unmounting filesystem on $K"
                        ( 
                        result=$(umount ${TMP_SG_FS_UMOUNT_OPT[$i]} $K 2>&1)
                        ret=$?
                        if (( ret != 0 ))
                        then
                            case $result in
                                *not*mounted*)
                                (( ret = 0 ))
                                ;;
                            *)
                            sg_log 0 "WARNING: Running fuser to remove anyone using the file system directly."
                            ;;
                            esac
                        fi
                        UM_COUNT=$UM_CNT
                        while (( ret != 0 && $UM_COUNT > 0 ))
                        do					
                            fuser -kuvm $K
                        
                            if [[ $FSS != "" ]]
                            then
                                sg_check_nfsshare_ready ${TMP_SG_FS_SERVER[$i]} ${TMP_SG_FS_NAME[$i]}
                                ret=$?
                                # NFS Server is not accessible. 
                                # Do forcible unmount, hence set UMOUNT RETRY COUNT to 1.
                                if [[ $ret != 0 ]]
                                then
                                    UM_COUNT=1
                                else
                                    umount ${TMP_SG_FS_UMOUNT_OPT[$i]} $K
                                    ret=$?
                                fi
                            else
                                umount ${TMP_SG_FS_UMOUNT_OPT[$i]} $K
                                ret=$?
                            fi
						                       
                            if (( ret != 0 ))
                            then
                                (( UM_COUNT = UM_COUNT - 1 ))
                                if (( UM_COUNT > 0 ))
                                then
                                    sg_log 0 "WARNING: Unmount failed, trying again."
                                    sleep 1
                                elif [[ $FSS != "" ]]
                                then
                                    sg_log 0 "Unmounting file system $I using the -f option"
                                    umount -f ${TMP_SG_FS_UMOUNT_OPT[$i]} $I;
                                    ret=$?
                                    io_timeout=`cmviewcl -v -f line -s config 2> /dev/null | \
                                                grep configured_io_timeout_extension | \
                                                awk -F"=" '{ print $NF }'`
                                    if [ -z "$io_timeout" ]
                                    then
                                        sg_log 0 "cmviewcl FAILED, trying again"
                                        io_timeout=`cmviewcl -v -f line -s config 2> /dev/null | \
                                                    grep configured_io_timeout_extension | \
                                                    awk -F"=" '{ print $NF }'`

                                        if [ -z "$io_timeout" ]
                                        then
                                            sg_log 0 "cmviewcl FAILED again, Using 0 as default \
                                                      value for configured_io_timeout_extension"
                                            io_timeout=0
                                        fi
                                    fi

                                    # CONFIGURED_IO_TIMEOUT_EXTENSTION parameter defined in Cluster ascii file
                                    # has unit in micro-seconds. Convert it to seconds before use.
                                    let io_timeout=$io_timeout/1000000
                                    sg_log 0 "Waiting for $io_timeout seconds to complete any pending IO to $I"
                                    sleep $io_timeout
                                fi
                            fi
                        done
                   # Even after trying to unmount for FS_UMOUNT_RETRY_COUNT, unmount still fails.
                   # If the filesystem is VXFS, then do forcible unmount using vxumount -o force.
                   if [[ $ret != 0 ]]
                   then
                       if [[ ${TMP_SG_FS_TYPE[$i]} = "vxfs" ]]
                       then
                           sg_log 0 "Unmounting filesystem $K using the -o force option"
                           $VXUMOUNT -o force $K;
                           ret=$?
                           if [[ $ret != 0 ]]
                           then
                                sg_log 0 "ERROR: Cannot unmount $K using -o force option"
                           fi
                       fi
                   fi

                        return $ret
                        )&

                        # save the process id and name of logical volume to be used later
                        # while checking the exit status
                        pids_list[$mnts_identified]="$!"
                        volume_list[$mnts_identified]=$I
                        fs_dir_list[$mnts_identified]=$K
                        (( mnts_identified = mnts_identified + 1 ))
                    fi
                fi
                    (( mnts_processed = mnts_processed + 1 ))
            fi #new
        done

        # wait for background umount processes to finish
        while (( mnts_identified  > 0 ))
        do
            pid=${pids_list[$mnts_identified-1]}
            I=${volume_list[$mnts_identified-1]}
            K=${fs_dir_list[$mnts_identified-1]}
            wait $pid
            if (( $? != 0 ))
            then
                if sg_check_if_it_is_reconfiguration
                then
                    sg_log 999 "ERROR: Failed to unmount $I on $K"
                    sg_set_reconfig_failure
                fi
                sg_log 0 "ERROR: Function sg_umount_fs"
                sg_log 0 "ERROR: Failed to unmount $I"
                exit_value=1
            else
                if sg_check_if_it_is_reconfiguration
                then
                    sg_log 999 "SUCCESS: Successfully unmounted $I on $K"
                fi
            fi
            (( mnts_identified  = mnts_identified  - 1 ))
        done
    done

# Check exit value (set if any preceeding mount calls failed)
if (( exit_value == 1 ))
then
    to_exit=1
    return
fi
}

# This function computes deleted/modified
# file systems from the previous package configuration.
# It goes through all the file systems and its options
# from the previous configuration and compares them
# against the current configuration. If it doesn't find
# FS or its associated attributes like FS directory
# or umount options, then the function marks them as deleted
# filesystems and then calls sg_umount_fs() with the constructed
# input parameters. Filesystems are unmounted in sg_umount_fs function

function sg_reconfigure_stop_fs
{
    declare -a TMP_SG_FS_NAME
    declare -a TMP_SG_FS_SERVER 
    declare -a TMP_SG_FS_UMOUNT_OPT
    declare -a TMP_SG_FS_DIRECTORY
    declare -a TMP_SG_FS_TYPE
    declare -a TMP_PREV_SG_FS_DETAIL
    declare -a TMP_CURR_SG_FS_DETAIL
    typeset -i TMP_SG_CONCURRENT_MOUNT_AND_UMOUNT_OPERATIONS
    typeset -i num_fs
    typeset -i a_num_fs
    typeset -i fs
    typeset -i a_fs
    typeset -i count=0

    sg_log 5 "sg_reconfigure_stop_fs"

    # Refresh the mount command output
    cache_mount_output

    a_num_fs=${#PREV_SG_FS_NAME[*]}
    a_fs=0

    num_fs=${#SG_FS_NAME[*]}
    fs=0
    while (( $fs < $num_fs )) 
    do 
        _temp="${SG_FS_NAME[$fs]}+${SG_FS_DIRECTORY[$fs]}+
              ${SG_FS_TYPE[$fs]}+${SG_FS_SERVER[$fs]}+
              ${SG_FS_MOUNT_OPT[$fs]}"
        TMP_CURR_SG_FS_DETAIL[$fs]=`echo $_temp|sed -e 's/[[:space:]]*$/+/'`
        (( fs = fs + 1 ))
    done

    a_fs=0
    while (( $a_fs < $a_num_fs ))
    do
        _temp="${PREV_SG_FS_NAME[$a_fs]}+${PREV_SG_FS_DIRECTORY[$a_fs]}+
              ${PREV_SG_FS_TYPE[$a_fs]}+${PREV_SG_FS_SERVER[$a_fs]}+
              ${PREV_SG_FS_MOUNT_OPT[$a_fs]}"
        PREV_SG_FS_DETAIL=`echo $_temp|sed -e 's/[[:space:]]*$/+/'`
        echo ${TMP_CURR_SG_FS_DETAIL[@]}|grep -q "$PREV_SG_FS_DETAIL"
        if [[ $? != 0 ]] 
        then
            TMP_SG_FS_NAME[$count]=${PREV_SG_FS_NAME[$a_fs]}
            TMP_SG_FS_SERVER[$count]=${PREV_SG_FS_SERVER[$a_fs]}
            TMP_SG_FS_UMOUNT_OPT[$count]=${PREV_SG_FS_UMOUNT_OPT[$a_fs]}
            TMP_SG_FS_DIRECTORY[$count]=${PREV_SG_FS_DIRECTORY[$a_fs]}
            TMP_SG_FS_TYPE[$count]=${PREV_SG_FS_TYPE[$a_fs]}
            TMP_SG_CONCURRENT_MOUNT_AND_UMOUNT_OPERATIONS=$PREV_SG_CONCURRENT_MOUNT_AND_UMOUNT_OPERATIONS
            (( count = count + 1 ))
        fi
        (( a_fs = a_fs + 1 ))
    done

    if (( ${#TMP_SG_FS_NAME[@]} > 0 ))
    then
        if sg_check_if_reconfig_has_failed
        then
            for I in ${TMP_SG_FS_NAME[@]}
            do
                sg_log 999 "WARNING: Not attempting to unmount "$I
                to_exit=1
            done
        else
            sg_umount_fs
        fi
    fi
}

# This function computes added/modified
# file systems to the current package configuration.
# It goes through all the file systems and its options
# from the current configuration and compares them
# against the previous configuration. If it doesn't find
# FS or its associated attributes like FS directory, fs type
# or umount options, etc, then the function marks them as added
# filesystems and then calls sg_check_and_mount() with the constructed
# input parameters. Filesystems are mounted in sg_check_and_mount() function


function sg_reconfigure_start_fs
{
    declare -a TMP_SG_FS_NAME
    declare -a TMP_SG_FS_SERVER
    declare -a TMP_SG_FS_DIRECTORY
    declare -a TMP_SG_FS_TYPE
    declare -a TMP_SG_FS_MOUNT_OPT
    declare -a TMP_SG_FS_FSCK_OPT
    typeset -i TMP_SG_CONCURRENT_MOUNT_AND_UMOUNT_OPERATIONS
    typeset -i num_fs
    typeset -i a_num_fs
    typeset -i fs
    typeset -i a_fs
    typeset -i count=0


    sg_log 5 "sg_reconfigure_start"

    # Refresh the mount command output
    cache_mount_output

    a_num_fs=${#PREV_SG_FS_NAME[*]}
    a_fs=0

    num_fs=${#SG_FS_NAME[*]}
    fs=0
    while (( $a_fs < $a_num_fs ))
    do
        _temp="${PREV_SG_FS_NAME[$a_fs]}+${PREV_SG_FS_DIRECTORY[$a_fs]}+
              ${PREV_SG_FS_TYPE[$a_fs]}+${PREV_SG_FS_SERVER[$a_fs]}+
              ${PREV_SG_FS_MOUNT_OPT[$a_fs]}"
        TMP_PREV_SG_FS_DETAIL[$a_fs]=`echo $_temp|sed -e 's/[[:space:]]*$/+/'`
        (( a_fs = a_fs + 1 ))
    done

    fs=0

    while (( $fs < $num_fs ))
    do
        _temp="${SG_FS_NAME[$fs]}+${SG_FS_DIRECTORY[$fs]}+
              ${SG_FS_TYPE[$fs]}+${SG_FS_SERVER[$fs]}+
              ${SG_FS_MOUNT_OPT[$fs]}"
        SG_FS_DETAIL=`echo $_temp|sed -e 's/[[:space:]]*$/+/'`
        echo ${TMP_PREV_SG_FS_DETAIL[@]}|grep -q "$SG_FS_DETAIL"
        if [[ $? != 0 ]]
        then
            if ! sg_check_if_reconfig_has_failed
            then
                sg_log 0 "FS added or changed ${SG_FS_NAME[$fs]}"
            fi
            TMP_SG_FS_NAME[$count]=${SG_FS_NAME[$fs]}
            TMP_SG_FS_SERVER[$count]=${SG_FS_SERVER[$fs]}
            TMP_SG_FS_MOUNT_OPT[$count]=${SG_FS_MOUNT_OPT[$fs]}
            TMP_SG_FS_DIRECTORY[$count]=${SG_FS_DIRECTORY[$fs]}
            TMP_SG_FS_TYPE[$count]=${SG_FS_TYPE[$fs]}
            TMP_SG_FS_FSCK_OPT[$count]=${SG_FS_FSCK_OPT[$fs]}
            TMP_SG_CONCURRENT_MOUNT_AND_UMOUNT_OPERATIONS=$SG_CONCURRENT_MOUNT_AND_UMOUNT_OPERATIONS
            (( count = count + 1 ))
        fi
        (( fs = fs + 1 ))
    done

    if (( ${#TMP_SG_FS_NAME[@]} > 0 ))
    then
        if sg_check_if_reconfig_has_failed
        then
            for I in ${TMP_SG_FS_NAME[@]}
            do
                sg_log 999 "WARNING: Not attempting to mount "$I
            done
        else
            sg_check_and_mount
        fi
    fi
}

function sg_start_check_and_mount
{
    declare -a TMP_SG_FS_NAME
    declare -a TMP_SG_FS_SERVER
    declare -a TMP_SG_FS_DIRECTORY
    declare -a TMP_SG_FS_TYPE
    declare -a TMP_SG_FS_MOUNT_OPT
    declare -a TMP_SG_FS_UMOUNT_OPT
    declare -a TMP_SG_FS_FSCK_OPT
    typeset -i TMP_SG_CONCURRENT_MOUNT_AND_UMOUNT_OPERATIONS
    typeset -i R=0

    # Refresh the mount command output
    cache_mount_output

    if sg_check_if_reconfig_has_failed
    then
        for I in ${SG_FS_NAME[@]}
        do
            sg_log 999 "WARNING: Not attempting to mount "$I
        done
        to_exit=1
    fi
    while (($R < ${#SG_FS_NAME[*]}))
    do
        TMP_SG_FS_NAME[$R]=${SG_FS_NAME[$R]}
        TMP_SG_FS_SERVER[$R]=${SG_FS_SERVER[$R]}
        TMP_SG_FS_DIRECTORY[$R]=${SG_FS_DIRECTORY[$R]}
        TMP_SG_FS_TYPE[$R]=${SG_FS_TYPE[$R]}
        TMP_SG_FS_MOUNT_OPT[$R]=${SG_FS_MOUNT_OPT[$R]}
        TMP_SG_FS_UMOUNT_OPT[$R]=${SG_FS_UMOUNT_OPT[$R]}
        TMP_SG_FS_FSCK_OPT[$R]=${SG_FS_FSCK_OPT[$R]}
        TMP_SG_CONCURRENT_MOUNT_AND_UMOUNT_OPERATIONS=$SG_CONCURRENT_MOUNT_AND_UMOUNT_OPERATIONS

        (( R = R + 1 ))
    done

    sg_check_and_mount
}

function sg_stop_umount_fs
{
    declare -a TMP_SG_FS_NAME
    declare -a TMP_SG_FS_UMOUNT_OPT
    declare -a TMP_SG_FS_SERVER
    declare -a TMP_SG_FS_DIRECTORY
    declare -a TMP_SG_FS_TYPE
    declare -a TMP_SG_CONCURRENT_MOUNT_AND_UMOUNT_OPERATIONS
    typeset -i R=0

    # Refresh the mount command output
    cache_mount_output

    while (($R < ${#SG_FS_NAME[*]}))
    do
        TMP_SG_FS_NAME[$R]=${SG_FS_NAME[$R]}
        TMP_SG_FS_UMOUNT_OPT[$R]=${SG_FS_UMOUNT_OPT[$R]}
        TMP_SG_FS_DIRECTORY[$R]=${SG_FS_DIRECTORY[$R]}
        TMP_SG_FS_SERVER[$R]=${SG_FS_SERVER[$R]}
        TMP_SG_FS_TYPE[$R]=${SG_FS_TYPE[$R]}
        TMP_SG_CONCURRENT_MOUNT_AND_UMOUNT_OPERATIONS=$SG_CONCURRENT_MOUNT_AND_UMOUNT_OPERATIONS

        (( R = R + 1 ))
    done

    sg_umount_fs
}

function cache_mount_output 
{
    MOUNT_CMD_OUTPUT=($(mount 2>&1))
}

################
# main routine
################

#
# Module script must be specified with three required entry points:
# start, stop, and validate.
#
# The variable to_exit indicates the success or failure of the entry 
# point.
#

typeset -i i=0 
typeset -i to_exit=0

sg_log 5 "sg_filesystems"

MOUNT_CMD_OUTPUT=""

case ${1} in
     start)      
           sg_start_check_and_mount
           ;;
     stop)      
           sg_stop_umount_fs
           ;;
     validate)  
           sg_validate_filesystems
           ;;
     reconfigure_stop)
         sg_reconfigure_stop_fs
          ;;
    reconfigure_start)
         sg_reconfigure_start_fs
          ;;
     *)
           sg_log 0 "INFO: Unknown operation - $1"
           ;;
esac

#
# Exit to indicate success/failure to the master_control_script.sh
#
exit $to_exit
