####################################################################
# (C) Copyright 2008-2015 Hewlett Packard Enterprise Development LP.
# @(#) Serviceguard UPCC Package Configuration Script.
# @(#) Product Name                :  HP Serviceguard 
# @(#) Product Version             :  A.12.10.00  
# @(#) Patch Name                  :  
#
# *** Note: This file MUST NOT be edited. *****
#
# Any changes made to it will be overwritten when you upgrade to the
# next release of HP Serviceguard.
#
# Changing this file may lead to unrecoverable package startup
# and/or shutdown problems.
#
#####################################################################

#############################################################################
# NAME:    pr_cntl
#
# SYNOPSIS:
#    pr_cntl [ start | stop | validate | reconfig ]
#
#    start     Execute package startup and perform Persistent reservations
#    stop      Execute package stop or failover and delete the PR key
#              from the underlying storage devices of the package.
#    validate  Execute a pre-startup check to validate if storage 
#              devices support Persistent Reservations.
#    reconfig  Execute a reconfig to see if any new VGs have been 
#              added or underlying LUNs added to existing VGs
#
# DESCRIPTION:
# This is the script called by the UPCC Master control script to set
# or revoke the persistent reservations of the storage devices accessed 
# by the package applications.
# It is also called by the cmapplyconf to "validate" if the package
# can use Persistent Reservations.
# It calls the pr_script pr_util to actually perform the Persistent 
# Reservations operations.
# For "start" and "stop" operations it also starts or terminates the 
# the MNP daemon.
# The package environmental variables that are accessed include
#
# SG_PACKAGE_NAME - The name of the package
# SG_PACKAGE_TYPE - The type of package, failover or MNP 
# SG_PR_MODE      - The Persistent Reservation mode,
#                   Either "pr_enabled" - package will use PR
#                          "pr_disabled" - package will not use PR
#
############################################################################# 

###########################
# Source utility functions.
###########################

if [[ -f ${SG_UTILS} ]]; 
then
    . ${SG_UTILS}
    if (( $? != 0 ))
    then
        echo "ERROR: Unable to source package utility functions file: ${SG_UTILS
}"
        exit 1
    fi
else
    echo "ERROR: Unable to find package utility functions file: ${SG_UTILS}"
    exit 1
fi

###################################################################
#
# Get the environment for this package through utility function
# sg_source_pkg_env().
#
###################################################################

sg_source_pkg_env $*

#########################
# Main processing routine
#########################
#sg_log 5 "pr_cntl"

# Let's just set a large timeout value 
#
typeset -i TIMEOUT=4000
typeset -i RET=0
typeset -i PR_STATUS=0 
typeset -i RETVAL=0
typeset -i skip_pr_module=0

CMD=$1

PR_UTIL=${SGCONF}/scripts/sg/pr_util.sh

case $CMD in
    start)
        if sg_check_if_pr_module_required; then
            skip_pr_module=0
        elif sg_check_if_skip_pr_modules; then
            skip_pr_module=1
        fi
        if (( skip_pr_module == 0 ))
        then
            if [ "$PKG_PR_MODE" = "pr_enabled" ]
            then
                $PR_UTIL $CMD
                RETVAL=$?
                if [ $RETVAL -eq 2 ]
                then
                    RET=2
                elif [ $RETVAL -ne 0 ]
                then
                    RET=1
                else
                    RET=0
                fi
            fi
        else
            sg_log 5 "INFO: Package configured with VMFS storage. PR module not required."
            RET=0
        fi
        ;;
    stop)
        if sg_check_if_skip_pr_modules; then
            skip_pr_module=1
        fi
        if (( skip_pr_module == 0 ))
        then
            if [ "$PKG_PR_MODE" = "pr_enabled" ]
            then
                $PR_UTIL $CMD
                RETVAL=$?
                if [ $RETVAL -eq 2 ]
                then
                    RET=2
                elif [ $RETVAL -ne 0 ]
                then
                    RET=1
                else
                    RET=0
                fi
            fi
        else
            sg_log 5 "INFO: Package configured with VMFS storage. PR module not required."
            RET=0
        fi
        ;;
    validate)
        if sg_check_if_storage_module_skip
        then
            sg_log 5 "INFO: Package configured with VMFS storage. PR module not required."
            RET=0
        elif sg_check_if_skip_pr_modules
        then
            sg_log 5 "INFO: Package configured with VMFS storage. PR module not required."
            RET=0
        else
            $PR_UTIL $CMD
            RET=$?
            if [ $RET -eq 0 ]
            then
                sg_log 0 "PR_mode:ENABLED"
            else
                if [ $RET -eq 3 ]
                then
                    sg_log 0 "PR_mode:DISABLED"
                    RET=0
                else
                    sg_log 0 "PR_mode:DISABLED"
                fi
            fi
        fi
        ;;
    reconfigure_start)
        # NOTE DO NOT DELETE THESE COMMENTS
        # Adding the string "sg_check_if_it_is_reconfiguration" and 
        # "sg_check_if_reconfig_has_failed". This is to support the
        # new online reconfiguration behavior. These strings are added
        # as comments. This ensures the check from master_control_script
        # that this module can handle the new behavior during online 
        # configuration.        
        if sg_check_if_pr_module_required; then
            skip_pr_module=0
        elif sg_check_if_skip_pr_modules; then
            skip_pr_module=1
        fi
        if (( skip_pr_module == 0 ))
        then
            if [ "$PKG_PR_MODE" = "pr_enabled" ]
            then
                $PR_UTIL $CMD
                RET=$?
                if [ $RET -eq 3 ]
                then
                    RET=0
                fi
            fi
        else
            sg_log 5 "INFO: Package configured with VMFS storage. PR module not required."
            RET=0
        fi
        ;;
    reconfigure_stop)
        if sg_check_if_skip_pr_modules; then
            skip_pr_module=1
        fi
        if (( skip_pr_module == 0 ))
        then
            $PR_UTIL $CMD
            RET=$?
            if [ $RET -eq 3 ]
            then
                RET=0
            fi
        else
            sg_log 5 "INFO: Package configured with VMFS storage. PR module not required."
            RET=0
        fi
        ;;
    reconfig)
        if [ $PKG_PR_MODE = "pr_enabled" ]
        then
            $PR_UTIL $CMD
            RET=$?
        fi
        ;;
        
    *)
        sg_log 0 "INFO:Unknown operation $CMD"
        ;;
esac

exit $RET
