####################################################################
# (C) Copyright 2014 Hewlett-Packard Development Company, L.P.
# @(#) Serviceguard UPCC Package Configuration Script.
# @(#) Product Name                :  HP Serviceguard
# @(#) Product Version             :  A.12.10.00
# @(#) Patch Name                  :  
#
# *** Note: This file MUST NOT be edited. *****
#
# Any changes made to it will be overwritten when you upgrade to the
# next release of HP Serviceguard.
#
# Changing this file may lead to unrecoverable package startup
# and/or shutdown problems.
#
###################################################################

# This script can be invoked from inside or outside of master_control_script.
# It can be invoked outside master_control_script in the following way:
# ${SGCONF}/scripts/sg/sc.sh "start" "<site_name>" "<Site_controller_package_name>"

#########################################
# Source Site Controller utility functions.
##########################################
. /etc/cmcluster.conf

SC_UTILS=${SGCONF}/scripts/sg/sc_util.sh
export SC_UTILS

# Set PATH to reference the appropriate directories.
PATH=$SGSBIN:/bin:/sbin:/usr/bin:/usr/sbin

if [[ -f ${SC_UTILS} ]]; then
    . ${SC_UTILS}
    if (( $? != 0 ))
    then
        echo "ERROR: Unable to source package Site Controller utility functions file: ${SC_UTILS}"
        exit 1
    fi
else
    echo "ERROR: Unable to find Site Controller utility functions file: ${SC_UTILS}"
    exit 1
fi

###################################################################
#
# Get the environment for this package through utility function
# sg_source_pkg_env().
#
###################################################################

postfix=$(date +"%H.%M.%S")
typeset env_file=$sgsadta_tmp_dir/pkg_env_${postfix}_$$

if [ "${3}" != "" ]
then
    ${SGSBIN}/cmgetpkgenv ${3} > ${env_file}
    sg_source_pkg_env "start" ${env_file}
else
    sg_source_pkg_env $*
fi

#######################################################################
#
# function sg_start_sc() is called by master control script
# during site controller package startup.
#
#######################################################################

function sg_start_sc
{
    typeset sc
    typeset site_arg 
    typeset local_site
    typeset remote_site
    typeset local_site_mode
    typeset remote_site_mode
    typeset new_remote_site
    typeset -i ret=1
    
    sg_log 5 "Function sg_start_sc"
    
    if [ -z $1 ]; then
       site_arg=""
    else
       site_arg=$1
    fi
               
    sc=`echo ${SG_PACKAGE_NAME}`
    setup_local_and_remote_site $site_arg 

    if [[ -z "$local_site" ]]
    then
        sg_log 0 "Failed to get local site name, cluster might be in transient state. Exiting."
        to_exit=1
        return 
    fi
    
    # Fail if not able to query the remote site name
    if [[ -z "$remote_site" ]]
    then
       sg_log 0 "Failed to get remote site name, cluster might be in transient state. Exiting."
       to_exit=1
       return
    fi

    sg_log 0 "Starting Site Controller ($sc) on site $local_site"
    sys_log "Starting Site Controller ($sc) on site $local_site"
    sg_log 0 "Local Site: $local_site, Remote Site: $remote_site"
 
    #get local site mode, it can be active, passive, remote_passive or intermediate
    local_site_mode=`sc_get_site_mode $sc $local_site`
    if [ "$local_site_mode" = "error" ]
    then
        sg_log 0 "Unable to find the site safety latch value at site: \"$local_site\" "
        to_exit=1
        return
    fi
    
    remote_site_mode=`sc_get_site_mode $sc $remote_site`
    if [ "$remote_site_mode" = "error" ]
    then
        sg_log 0 "Unable to find the site safety latch value at site: \"$remote_site\" "
        to_exit=1
        return
    fi
    
    #If the local or the remote site is in any of the following modes
    #site controller cannot start unless a user resets the site using cmresetsc
    # Pseudo-passive state, Intermediate state
    if [[ "$local_site_mode" = "${INTERMEDIATE_STATE}" ]] \
        || [[ "$remote_site_mode" = "${INTERMEDIATE_STATE}" ]] \
        || [[ "$local_site_mode" = "${PSEUDO_PASSIVE_STATE}" ]] \
        || [[ "$remote_site_mode" = "${PSEUDO_PASSIVE_STATE}" ]] 
    then
            sc_print_reset_message $local_site_mode $remote_site_mode $sc
            to_exit=1
            return
    fi

    #Error scenarios: This should never happen ideally. Making sure we will never 
    #                 proceed in these scenarios.
    #                 Both sites are active or both sites are in remote passive state
    if [[ "$local_site_mode" = "${ACTIVE_STATE}" && \
          "$remote_site_mode" = "${ACTIVE_STATE}" ]] || \
       [[ "$remote_site_mode" = "${REMOTE_PASSIVE_STATE}" && \
          "$local_site_mode" = "${REMOTE_PASSIVE_STATE}" ]] 
    then
        sg_log 0 "Both sites are in $local_site_mode state"
        sg_log 0 "Clean up both the sites and start $sc again"
        sg_log 0 "Check for any error messages in the package log file on all nodes"
        sg_log 0 "Fix any issue reported in the package log files and enable "
        sg_log 0 "node switching for the packages on nodes that have failed."
        sg_log 0 "Reset both the sites using \"$SGSBIN/cmresetsc\" command and "
        sg_log 0 "start $sc again"
        to_exit=1
        return
    fi

    #Scenario 1: SC and workload is running on active site. If failover of SC 
    #            happens on same site node and remote site is Remote passive 
    #            state then, do nothing.
    #Scenario 2: SC is running on passive site, workload is running on active site. 
    #            If failover of SC happens on another passive site node then, 
    #            do nothing.
    if [[ "$local_site_mode" = "${ACTIVE_STATE}" && \
          "$remote_site_mode" = "${REMOTE_PASSIVE_STATE}" ]] || \
       [[ "$local_site_mode" = "${REMOTE_PASSIVE_STATE}" && \
          "$remote_site_mode" = "${ACTIVE_STATE}" ]] 
    then
        sg_log 0 "Site Controller startup succeeded."
        to_exit=0
        return
    fi
   
    #Scenario 1: Previous startup of Site Controller, one of sites was in transient state.
    #After resetting the transient site, Site Controller startup is issued.
    #In this case, we should set the passive state site to remote passive state
    #so that remote managed packages can start.
    #Scenario 2: When SC is in detached mode and cluster is halted, the Generic
    # resource value on the SC node does not get reset but retains its previous value. 
    # So, when SC is halted and run again, it gets the previous active site (site where 
    # the previously detached node GR status was active) and tries to startup 
    # on another node in the same site.
    
    if [[ "$local_site_mode" = "${ACTIVE_STATE}" && \
          "$remote_site_mode" = "${PASSIVE_STATE}" ]] || \
       [[ "$remote_site_mode" = "${ACTIVE_STATE}" && \
          "$local_site_mode" = "${PASSIVE_STATE}" ]] 
    then
        if [[ "$local_site_mode" = "${PASSIVE_STATE}" ]]
        then
            new_remote_site=`echo $local_site`
            new_local_site=`echo $remote_site`
        else
            new_remote_site=`echo $remote_site`
            new_local_site=`echo $local_site`
        fi

        sc_deactivate_site $sc $new_remote_site ${PSEUDO_PASSIVE_STATE}
        if [[ $? -eq 1 ]]
        then
            sg_log 0 "Failed to deactivate site $new_remote_site."
            sg_log 0 "Site Controller startup failed"       
            to_exit=1
            return
        fi
                 
        sc_activate_site $sc $new_local_site
        if [[ $? -eq 1 ]]
        then
            sg_log 0 "Check for any error messages in the package log file on all nodes"
            sg_log 0 "in the site \"$new_local_site\" for the package(s) managed by"
            sg_log 0 "Site Controller ( $sc )."
            sg_log 0 "Fix any issue reported in the package log files and "
            sg_log 0 "enable node switching for the package(s) on nodes that have failed."
            sg_log 0 "Reset the site \"$new_local_site\" using \"$SGSBIN/cmresetsc\" command"
            sg_log 0 "and start $sc again. Site Controller startup failed."
            to_exit=1
            return
        fi
        
        #get the nodes which are up in the current site
        site_nodes=`get_site_up_nodes $new_remote_site`
        if [[ -n "$site_nodes" ]]; then
            sc_remote_managed_pkg_start $sc $new_remote_site
               
            if [[ $? -eq 1 ]]
            then
                sg_log 0 "Check for any error messages in the remote managed package" 
                sg_log 0 "log file on all nodes in the site: \"$new_remote_site\"."
                sg_log 0 "Site Controller could not start remote managed package(s)"
                sg_log 0 "Fix any issue reported in the package log files,"
                sg_log 0 "enable node switching for the packages on nodes that "
                sg_log 0 "have failed and manually start the packages"
            fi
        fi
        
        sg_log 0 "Site Controller startup succeeded."
        to_exit=0
        return
     fi
    
    #This scenario will be hit in case of ungraceful inter_site
    #failover 
    #Scenario 1: Local site is in REMOTE_PASSIVE_STATE and 
    #            Remote site is in PASSIVE_STATE
    #Scenario 2: Remote site is in REMOTE_PASSIVE_STATE and 
    #            Local site is in PASSIVE_STATE
    
    if [[ "$local_site_mode" = "${REMOTE_PASSIVE_STATE}" && \
          "$remote_site_mode" = "${PASSIVE_STATE}" ]] || \
       [[ "$remote_site_mode" = "${REMOTE_PASSIVE_STATE}" && \
          "$local_site_mode" = "${PASSIVE_STATE}" ]] 
    then
        if [[ "$local_site_mode" = "${REMOTE_PASSIVE_STATE}" ]]
        then
            new_remote_site=`echo $local_site`
        else
            new_remote_site=`echo $remote_site`
        fi
        
        sc_deactivate_site $sc $new_remote_site ${PSEUDO_PASSIVE_STATE}
        if [[ $? -eq 1 ]]
        then
            sg_log 0 "Site Controller startup failed"       
            to_exit=1
            return
        fi        
    
        #since we set the site_safety_latch value at new_remote_site
        #Need to refresh local_site_mode and remote_site_mode     
        local_site_mode=`sc_get_site_mode $sc $local_site`
        if [ "$local_site_mode" = "error" ]
        then
            sg_log 0 "Unable to find the site safety latch value at site: \"$local_site\" "
            to_exit=1
            return
        fi

        remote_site_mode=`sc_get_site_mode $sc $remote_site`
        if [ "$remote_site_mode" = "error" ]
        then
            sg_log 0 "Unable to find the site safety latch value at site: \"$remote_site\" "
            to_exit=1
            return
        fi
    fi
    
    #At this stage, the only possible states can be
    #Scenario 1: Local site : PASSIVE_STATE         ; Remote Site: PASSIVE_STATE
    #Scenario 2: Local site : PASSIVE_STATE         ; Remote Site: PSEUDO_PASSIVE_STATE
    #Scenario 3: Local site : PSEUDO_PASSIVE_STATE  ; Remote Site: PASSIVE_STATE
    
    if [[ "$local_site_mode" = "${PASSIVE_STATE}" && \
          "$remote_site_mode" = "${PASSIVE_STATE}" ]] || \
       [[ "$local_site_mode" = "${PSEUDO_PASSIVE_STATE}" && \
          "$remote_site_mode" = "${PASSIVE_STATE}" ]] || \
       [[ "$local_site_mode" = "${PASSIVE_STATE}" && \
          "$remote_site_mode" = "${PSEUDO_PASSIVE_STATE}" ]] 
    then
        sg_log 0 "Initiating Site Controller startup at site $local_site"
    
        if [[ "$remote_site_mode" != "${PSEUDO_PASSIVE_STATE}" ]]
        then
            sc_deactivate_site $sc $remote_site ${PSEUDO_PASSIVE_STATE}
            if [[ $? -eq 1 ]]
            then
                sg_log 0 "Failed to deactivate site $remote_site."
                sg_log 0 "Site Controller startup failed"       
                to_exit=1
                return
            fi
         fi
         
         sc_activate_site $sc $local_site
         if [[ $? -eq 1 ]]
         then
             sg_log 0 "Check for any error messages in the package log file on all nodes"
             sg_log 0 "in the site \"$local_site\" for the package(s) managed by"
             sg_log 0 "Site Controller ( $sc )."
             sg_log 0 "Fix any issue reported in the package log files and "
             sg_log 0 "enable node switching for the package(s) on nodes that have failed."
             sg_log 0 "Reset the site \"$local_site\" using \"$SGSBIN/cmresetsc\" command"
             sg_log 0 "and start $sc again. Site Controller startup failed."
             to_exit=1
             return
         fi

         sc_remote_managed_pkg_start $sc $remote_site         
         if [[ $? -eq 1 ]]
         then
             sg_log 0 "Check for any error messages in the remote managed package"
             sg_log 0 "log file on all nodes in the site \"$remote_site\" for the package(s)."
             sg_log 0 "Site Controller could not start remote managed package(s)"
             sg_log 0 "Fix any issue reported in the package log files,"
             sg_log 0 "enable node switching for the package(s) on nodes that have"
             sg_log 0 "failed and manually start the package(s)"
             return
         fi    
    fi
    
    sg_log 0 "Site Controller $sc startup succeeded."
    sys_log "Site Controller $sc startup succeeded."

    to_exit=0
    return   
}


#######################################################################
#
# function sg_stop_sc() is called by master control script
# during site controller package stop.
#
#######################################################################

function sg_stop_sc
{
    typeset sc=${SG_PACKAGE_NAME}
    typeset halt_reason=$SG_HALT_REASON
    typeset local_site
    typeset detach_flag="${SGRUN}/log/${SG_PACKAGE_NAME}_DETACH"

    sg_log 5 "Function sg_stop_sc"
    sg_log 5 "Serviceguard has set package halt_reason=$halt_reason"

    populate_already_identified_local_and_remote_site
    
    #In case where no site is active, set the site where local host 
    #belongs as local_site
    if [ -z "${local_site}" ]
    then
        setup_local_and_remote_site ""
    fi

    sg_log 5 "Function sg_stop_sc on local site $local_site"
    
    if [ "$halt_reason" = "user_halt" ]
    then
        #check the detach flag's presence
        if [ -e $detach_flag ]
        then
            sg_log 0 "Site Controller detach flag file $detach_flag found"
            rm -f $detach_flag
            if [[ $? -eq 1 ]];then
                sg_log 0 "Failed to delete Site Controller detach flag file"
                sg_log 0 "$detach_flag"
                sg_log 0 "Site Controller halt failed"
                to_exit=1
                return
            fi

            sg_log 0 "Site Controller halted successfully"
            to_exit=0
            return
        fi

        #regular SC halt
        #deactivate remote site
        sc_deactivate_site $sc $remote_site ${PASSIVE_STATE}
        if [[ $? -eq 1 ]];then
           sg_log 0 "Site Controller halt failed"
           to_exit=1
           return
        fi
        
        #deactivate local site
        sc_deactivate_site $sc $local_site ${PASSIVE_STATE}
        if [[ $? -eq 1 ]];then
           sg_log 0 "Site Controller halt failed"
           to_exit=1
           return
        fi
        
        sg_log 0 "Site Controller halted successfully"
        to_exit=0
        return
    else
        #halt reason is not user halt. 
        #cmcld has initiated package failover.
        to_exit=0
        return
    fi

}

#######################################################################
# is_valid_online_config determines whether the configuration changes
# are valid. Only the following changes are allowed online
# 1. Adding an Critical, Managed and remote managed package.
# 2. Removing an Critical, Managed and remote managed package.
# Following changes are not allowed
# 1. Addition and deletion of workload packages in a single cmapplyconf
# 2. Replacing an workload package in an single cmapplyconf.
#######################################################################

function is_valid_online_config
{
    typeset sc_name=$1
 
    sg_log 5 "Function is_valid_online_config $sc_name"  
    
    populate_already_identified_local_and_remote_site

    cache_prev_pkg_info $sc_name

    sc_get_removed_pkg_list
    sc_get_added_pkg_list
    
    if (( $remove_flag == 1 && $add_flag == 1 ))
    then
        sg_log 0 "ERROR: Online addition of workload package(s)($(IFS=, ; echo "${added_pkg_list[*]}")) and removal of" \
                 "workload package(s)($(IFS=, ; echo "${removed_pkg_list[*]}")) together is not supported."
        sg_log 0 "Please perform these two steps one after the other."
        to_exit=1
        return 
    fi

    # If local_site is not populated, don't need to check
    # if added packages are running.
    if [ -z "${local_site}" ]
    then
        return
    fi
    
    # Need to check if all the added workload packages which will be part of active 
    # workload are already running.

    for pkg in ${added_pkg_list_on_local_site[@]}
    do
        is_package_up $pkg
        if [[ $? -ne 0 ]]
        then
            sg_log 0 "ERROR: Online addition of $pkg which is not up is not supported."
            sg_log 0 "Start the package and ensure that $pkg is up and running before"
            sg_log 0 "adding it to the site controller $sc_name package configuration."
            to_exit=1
        fi
    done
    
}

#######################################################################
#
# function sg_validate_sc() is called by master control script
# during site controller package validation.
#
#######################################################################

function sg_validate_sc
{
    typeset -i retVal=1
    typeset sc=${SG_PACKAGE_NAME}
    sg_log 5 "Function sg_validate_sc"
    sc_get_monitor_service
    if [[ -z "${monitoring_service_name}" ]]
    then
        sg_log 0 "ERROR: Site Controller monitoring service is not configured"
        to_exit=1 
        return
    fi

    sg_check_module_in_pkg $SGESAP_MODULE
    if [ $? -ne 0 ]; then    
        #Validate that the package  auto run is set to NO
        check_package_autorun
        retVal=$?
        if [[ $retVal -eq 1 ]]
        then
            to_exit=1
            return
        fi
    fi
    
    is_package_up $sc
    if [[ $? -eq 0 ]]
    then
        is_valid_online_config $sc
    fi
    return
}

###############################################################################		
# This function checks whether the sitecontroller autorun parameter is set to no		
#		
# Internal Use only		
#		
#		
# Arguments:  none		
#		
# Returns:    0 - if autorun is no		
#             1 - if autorun is yes		
#		
###############################################################################		
		
function check_package_autorun		
{		
    if [ $SG_AUTO_RUN = "yes" ]		
    then		
        sg_log 0 "ERROR: The auto_run flag should be set to NO"		
        return 1		
    fi		
    return 0		
}

#######################################################################
#
# function sg_sc_reconfig_stop_monitor_service() stops the site 
# controller monitoring service.
#
#######################################################################

function sg_sc_reconfig_stop_monitor_service
{
    if [[ -n "$monitoring_service_name" ]]
    then
        sg_log 0 "Stopping service ${monitoring_service_name}"
        ${SGSBIN}/cmhaltserv ${monitoring_service_name} >> $SG_SCRIPT_LOG_FILE 2>&1 
        if (( $? != 0 ))
        then
            sg_log 999 "ERROR: Failed to halt the service"
            sg_log 999 "${monitoring_service_name} for online package modification"
            sg_set_reconfig_failure
            to_exit=1
        fi
    fi
}

#######################################################################
#
# function sg_sc_reconfig_start_monitor_service() starts the site
# controller monitoring service.
#
#######################################################################

function sg_sc_reconfig_start_monitor_service
{
    if [[ -n "${monitoring_service_name}" ]]
    then
        sg_log 0 "Starting service ${monitoring_service_name}"
        ${SGSBIN}/cmrunserv ${monitoring_service_name} "${monitoring_service_cmd}" >> $SG_SCRIPT_LOG_FILE 2>&1
        if (( $? != 0 ))
        then
            sg_log 999 "ERROR: Failed to start the service"
            sg_log 999 "${monitoring_service_name} for online package modification"
            sg_set_reconfig_failure
            to_exit=1
        fi
    fi
}

#######################################################################
#
# function sg_sc_reconfig_restart_monitor_service() restarts the site
# controller monitoring service.
#
#######################################################################

function sg_sc_reconfig_restart_monitor_service
{
    if [[ -n "${monitoring_service_name}" ]]
    then
        sg_sc_reconfig_stop_monitor_service
        sg_sc_reconfig_start_monitor_service
        if ! sg_check_if_reconfig_has_failed
        then
            sg_log 0 "SUCCESS: Successfully restarted service ${monitoring_service_name}"
            sg_log 999 "SUCCESS: Successfully restarted service ${monitoring_service_name}"
        fi
    fi
}

#######################################################################
#
# function sg_reconfigure_stop_sc() is called by master control script
# during site controller package reconfigure_stop entry point.
#
#######################################################################

function sg_reconfigure_stop_sc
{
    sg_log 5 "Function sg_reconfigure_stop_sc"
   
    typeset -i need_to_restart=0
    typeset sc=${SG_PACKAGE_NAME}
   
    cache_prev_pkg_info $sc
 
    sc_get_removed_pkg_list 
    
    if [ "${removed_pkg_list[@]}" != "" ]
    then
        need_to_restart=1
    fi

    if ((need_to_restart == 0)); then
       if [[ "${PREV_SG_SC_SITE_MONITOR_INTERVAL}" != "${SG_SC_SITE_MONITOR_INTERVAL}" ]]; then
          need_to_restart=1 
       fi
    fi

    if (( need_to_restart == 1 ))
    then
        sc_get_monitor_service
        sg_sc_reconfig_restart_monitor_service            
    fi
}

#######################################################################
#
# function sg_reconfigure_start_sc() is called by master control script
# during site controller package reconfigure_stop entry point.
#
#######################################################################

function sg_reconfigure_start_sc
{
    typeset newfailoverpkglist
    typeset sc=${SG_PACKAGE_NAME}
   
    sg_log 5 "Function sg_reconfigure_start_sc" 
   
    postfix=$(date +"%H.%M.%S")
    temp_file=$sgsadta_tmp_dir/${SG_PACKAGE_NAME}_cmrunpkg_${postfix}_$$.tmpvalidate
    
    populate_already_identified_local_and_remote_site

    if [[ -z "$local_site" ]]
    then
        sg_log 999 "Failed to get local site name, cluster might be in transient state. Exiting."
        sg_set_reconfig_failure 
        to_exit=1
        return
    fi

    cache_prev_pkg_info $sc
    
    sc_get_added_pkg_list

    if [ "${added_pkg_list_on_local_site[@]}" != "" ]
    then
         sc_get_monitor_service
         sg_sc_reconfig_restart_monitor_service
    fi

    for pkg in ${added_pkg_list_on_local_site[@]}
    do
        is_failover_pkg ${pkg}
        if [[ $? -eq 0 ]]
        then
            if [ -z "$newfailoverpkglist" ]; then
                newfailoverpkglist=$pkg
            else
                newfailoverpkglist=$newfailoverpkglist" "$pkg
            fi
        fi
    done
    
    if sg_check_if_reconfig_has_failed
    then
        sg_log 999 "WARNING: Not attempting to enable global switching for the"
        sg_log 999 "packages managed by Site Controller ($sc) : ${newfailoverpkglist}"
    else
        if [[ -n "${newfailoverpkglist}" ]]; then
           sg_log 0 "Enabling global package switching for the failover packages managed"
           sg_log 0 "by Site Controller ($sc) at site $local_site : ${newfailoverpkglist}"
           for pkg in ${newfailoverpkglist}
           do
               if sg_check_if_reconfig_has_failed
               then
                   sg_log 999 "WARNING: Not attempting to enable global switching for the"
                   sg_log 999 "package ${pkg}"
               else 
                   sg_log 0 "Executing: cmmodpkg -e $pkg"
                   $cmmodpkg -e "$pkg" > $temp_file
                   ret=$?
                   /bin/chmod 644 $temp_file 2>/dev/null

                   if [[ "$ret" -eq 1 ]]; then
                       sg_log 999 "Warning: Could not enable global package switching for the"
                       sg_log 999 "failover package ${pkg} managed by Site Controller"
                       sg_log 999 "`cat $temp_file`"
                       sg_log 999 "Check the log files of the packages managed by"
                       sg_log 999 "Site Controller for more details."
                       sg_set_reconfig_failure
                       to_exit=1
                   else
                       sg_log 999 "Successfully enabled global package switching for the failover package"
                       sg_log 999 "$pkg managed by Site Controller ($sc) at site $local_site"
                   fi  
               fi
               /bin/rm -f $temp_file 2>/dev/null
           done
        fi
    fi
}

#######################################################################
#
# function cleanup_files_before_exit() is called before site controller 
# package exits. This function attempts to cleanup any temporary file
# created for its use
#
#######################################################################

function cleanup_files_before_exit
{
    # Remove Env file created to pass on to sg_source_pkg_env()
    if [ -f ${env_file} ]; then
        rm -f ${env_file}
    fi
}

################
# main routine
################

#
# Module script must be specified with following required entry points:
# start, stop, validate, reconfigure_start and reconfigure_stop.
#
# The variable to_exit indicates the success or failure of the entry
# point.
#

typeset -i to_exit=0
typeset genres_sc="${SG_GENERIC_RESOURCE_NAME[0]}"
sg_log 5 "Site Controller Module Script"

init_cache ${SG_PACKAGE_NAME}

case ${1} in
    start)
        # if any of the workload packages are in maintenance mode
        # then we cannot start the site controller package.
        #
        sg_check_any_workload_pkg_in_maintenance
        if [ $? -eq 0 ]; then
            sg_log 0 "Hence unable to start Site Controller package"
            sg_log 0 "Turn off the maintenance mode for these package(s)"            
            sg_log 0 "and then attempt to start ${SG_PACKAGE_NAME}."                        
            to_exit=1
            cleanup_files_before_exit
            exit $to_exit 
        fi

        # if any of the workload packages are in detached mode
        # then we cannot start the site controller package.
        #
        sg_check_any_workload_pkg_in_detached
        if [ $? -eq 0 ]; then
            sg_log 0 "Hence unable to start Site Controller package"
            sg_log 0 "Either halt these package(s) or re-attach the node"            
            sg_log 0 "and then attempt to start ${SG_PACKAGE_NAME}."                        
            to_exit=1
            cleanup_files_before_exit
            exit $to_exit 
        fi
        
        sg_check_module_in_pkg $SGESAP_MODULE
        if [ $? -eq 0 ]; then
            if [ "${2}" != "" ]; then              
                sg_start_sc "${2}"
            else                  
                has_genres_ever_set $genres_sc
                # If genres is not been set, do nothing.
                if [ $? -eq 0 ]; then
                    sg_start_sc ""
                fi
            fi
        else
            sg_start_sc ""
        fi       
        ;;
    stop)
        # When SC is in maintenance and if a halt is issued halt only SC
        # No need to halt Workloads.
        sc_state=`is_pkg_in_maintenance ${SG_PACKAGE_NAME}`
        if [[ $sc_state -eq 0 ]]; then
            sg_log 0 "Site controller package ${SG_PACKAGE_NAME} is in maintenance"
            sg_log 0 "mode, hence not attempting to halt any workload packages"
            to_exit=0
            cleanup_files_before_exit
            exit $to_exit 
        fi
    
        #if genres has never been set and sg_stop_sc has been called
        #we do nothing. This is the case wherin site controller package
        #was started and SGeSAP module took a while to return the site name
        #and in the mean time OPCODE stop was issued by Master control script
        #(Halt was issued).          
        sg_check_module_in_pkg $SGESAP_MODULE
        if [ $? -eq 0 ]; then
          has_genres_ever_set $genres_sc
          if [ $? -eq 0 ]; then
              sg_stop_sc
          fi
        else
          sg_stop_sc
        fi              
          ;;
    validate)
          sg_validate_sc 
          ;;
    reconfigure_start)
          sg_check_module_in_pkg $SGESAP_MODULE
          if [ $? -eq 0 ]; then
              has_genres_ever_set $genres_sc
              if [ $? -eq 0 ]; then
                  sg_reconfigure_start_sc
              fi
          else
              sg_reconfigure_start_sc
          fi        
          ;;
    reconfigure_stop)
          sg_check_module_in_pkg $SGESAP_MODULE
          if [ $? -eq 0 ]; then
              has_genres_ever_set $genres_sc
              if [ $? -eq 0 ]; then
                  sg_reconfigure_stop_sc
              fi
          else
              sg_reconfigure_stop_sc
          fi        
          ;;
    *)
          sg_log 0 "INFO: Unknown operation - $1 "
          to_exit=0          
          ;;
esac

#
# Exit to indicate success/failure to the master_control_script.sh
#
cleanup_files_before_exit
exit $to_exit
