####################################################################
# (C) Copyright 2006-2014 Hewlett-Packard Development Company, L.P.
# @(#) Serviceguard UPCC Package Configuration Script.
# @(#) Product Name                :  HP Serviceguard
# @(#) Product Version             :  A.12.10.00
# @(#) Patch Name                  :  
#
# *** Note: This file MUST NOT be edited. *****
#
# Any changes made to it will be overwritten when you upgrade to the
# next release of HP Serviceguard.
#
# Changing this file may lead to unrecoverable package startup
# and/or shutdown problems.
#
###################################################################

###########################
# Source utility functions.
###########################

if [[ -f ${SG_UTILS} ]]; then
    . ${SG_UTILS}
    if (( $? != 0 ))
    then
        echo "ERROR: Unable to source package utility functions file: ${SG_UTILS}"
        exit 1
    fi
else
    echo "ERROR: Unable to find package utility functions file: ${SG_UTILS}"
    exit 1
fi

###################################################################
#
# Get the environment for this package through utility function
# sg_source_pkg_env().
#
###################################################################

sg_source_pkg_env $*

##########################################################################
#
# sg_validate_services() is called while validating the package
# configuration (through cmcheckconf/cmapplyconf)
#
##########################################################################

function sg_validate_services
{
    typeset -i r=0
    typeset -i j=0
    typeset -i nested=0
    typeset service_cmd

    sg_log 5 "sg_validate_services"

    #
    # Check if service commands are executable
    #
    while (( r < ${#SG_SERVICE_CMD[*]} ))
    do
        #
        # Extract the service command filename
        #
        set ${SG_SERVICE_CMD[$r]} 
        service_cmd=$1
        
        #
        # verify service_cmd does not reside under a configured mount point
        #
        j=0
        nested=0
        
        [[ ! -d $service_cmd ]] && while (( j < ${#SG_FS_DIRECTORY[*]} ))
        do
            if [[ ${service_cmd#${SG_FS_DIRECTORY[$j]}} != $service_cmd ]]
            then
                nested=1
                break
            fi 
            (( j = j + 1 ))
        done
        
        (( nested == 0 )) && if [[ -d $service_cmd ]] || [[ ! -x $service_cmd ]]
        then
            sg_log 0 "ERROR: Function sg_validate_services"
            sg_log 0 "       Service command $service_cmd is not executable"
            to_exit=1
        fi
        (( r = r + 1 ))
    done

}

##########################################################################
#
# sg_start_services() is called while starting a package (cmrunpkg)
#
# For each service name
# call sg_start_service() function that actually runs the service
#
##########################################################################
function sg_start_services
{
    typeset -i i=0

    sg_log 5 "sg_start_services"
    while ((i < ${#SG_SERVICE_NAME[@]}))
    do
        if sg_check_if_reconfig_has_failed
        then
            sg_log 999 "WARNING: Not attempting to start service "${SG_SERVICE_NAME[$i]}
            to_exit=1
        else
            sg_start_service ${SG_SERVICE_NAME[$i]} "${SG_SERVICE_CMD[$i]}" \
                             ${SG_SERVICE_RESTART[$i]} $SG_SCRIPT_LOG_FILE
            retval=$?
        fi
        ((i=i+1))
    done
}

#################################################################
# This function starts services using cmrunserv using the input
# passed as arguments.
# Arguments passed are service name, service command,
# restart value and script log file.
# This function is called during the package start and during package
# reconfiguration.
################################################################
function sg_start_service
{
    typeset -i retval=0

    typeset service_name=$1
    typeset service_cmd=$2
    typeset service_restart=$3
    typeset script_log_file=$4

    sg_log 5 "sg_start_service"

    sg_log 0 "Starting service $service_name using \"$service_cmd\""
    sg_log 5 " restart count = \"$service_restart\""
    #
    # Check if cmrunserv should be called the old
    # way without a restart count.
    #
    if [[ "$service_restart" = "none" ]]
    then
        cmrunserv $service_name ">> $script_log_file 2>&1 $service_cmd"
        retval=$?
    else
        #
        # Do not attempt to restart when service_cmd does not exists or
        # does not have executable permission(JAGag12644)
        # Extract the service script name first from the SERVICE_CMD
        # and then check for the execute permission
        #
        if [[ ! -x ${service_cmd%% *} ]]
        then
            cmrunserv $service_name ">> $script_log_file 2>&1 $service_cmd"
            retval=$?
        else
            if [[ "$service_restart" = "unlimited" ]]
            then
                cmrunserv -R $service_name ">> $script_log_file 2>&1 $service_cmd"
                retval=$?
            else
                cmrunserv -r $service_restart $service_name ">> $script_log_file 2>&1 $service_cmd"
                retval=$?
            fi
        fi
    fi

    if (( retval != 0 ))
    then
        sg_log 0 "ERROR: Function sg_start_service"
        sg_log 0 "ERROR: Failed to start the $service_name"
        if sg_check_if_it_is_reconfiguration
        then
            sg_log 999 "ERROR: Failed to start the $service_name"
            sg_set_reconfig_failure
        fi
        to_exit=1
    else
        sg_log 999 "SUCCESS: Successfully started the $service_name"
    fi
    return $retval
}
function sg_reconfigure_start_services
{
    typeset -i found=0
    typeset -i i=0
    typeset -i j=0
    typeset -i retval=0

    sg_log 5 "sg_reconfigure_start_services"

    while ((i < ${#SG_SERVICE_NAME[@]}))
    do
        found=0
        j=0
        while [[ j -lt ${#PREV_SG_SERVICE_NAME[@]} ]]
        do
            if [[ ${SG_SERVICE_NAME[$i]} = ${PREV_SG_SERVICE_NAME[$j]} ]]
            then
                found=1
                if [[ "${SG_SERVICE_CMD[$i]}" != "${PREV_SG_SERVICE_CMD[$j]}" ]]
                then
                    if sg_check_if_reconfig_has_failed
                    then
                        sg_log 999 "WARNING: Not attempting to start service ${SG_SERVICE_NAME[$i]}"
                        to_exit=1
                    else
                        sg_start_service ${SG_SERVICE_NAME[$i]} \
                           "${SG_SERVICE_CMD[$i]}" ${SG_SERVICE_RESTART[$i]} \
                           ${SG_SCRIPT_LOG_FILE}
                        retval=$?
                    fi
                fi
                break
            fi
            ((j=j+1))
        done

        if ((found == 0))
        then
            if sg_check_if_reconfig_has_failed
            then
                sg_log 999 "WARNING: Not attempting to start service ${SG_SERVICE_NAME[$i]}"
                to_exit=1
            else
                sg_start_service ${SG_SERVICE_NAME[$i]} \
                      "${SG_SERVICE_CMD[$i]}" ${SG_SERVICE_RESTART[$i]} \
                      ${SG_SCRIPT_LOG_FILE}
                retval=$?
            fi
        fi

        ((i=i+1))
    done
}

function sg_reconfigure_stop_services
{
    typeset -i found=0
    typeset -i retval=0
    typeset -i i=0
    typeset -i j=0

    sg_log 5 "sg_reconfigure_stop_services"

    while (($i < ${#PREV_SG_SERVICE_NAME[@]}))
    do
        found=0
        j=0
        while [[ $j -lt ${#SG_SERVICE_NAME[@]} ]]
        do
            if [[ ${SG_SERVICE_NAME[$j]} = ${PREV_SG_SERVICE_NAME[$i]} ]]
            then
                found=1
                if [[ "${SG_SERVICE_CMD[$j]}" != "${PREV_SG_SERVICE_CMD[$i]}" ]]
                then
                    if sg_check_if_reconfig_has_failed
                    then
                        sg_log 999 "WARNING: Not attempting to halt service ${PREV_SG_SERVICE_NAME[$i]}"
                        to_exit=1
                    else
                        sg_log 0 "Halting service ${PREV_SG_SERVICE_NAME[$i]}"
                        cmhaltserv ${PREV_SG_SERVICE_NAME[$i]} >> $PREV_SG_SCRIPT_LOG_FILE 2>&1
                        retval=$?
                    fi
                fi
                break
            fi
            ((j=j+1))
        done

        if ((found==0))
        then
            if sg_check_if_reconfig_has_failed
            then
                sg_log 999 "WARNING: Not attempting to halt service ${PREV_SG_SERVICE_NAME[$i]}"
                to_exit=1
            else
                sg_log 0 "Halting service ${PREV_SG_SERVICE_NAME[$i]}"
                cmhaltserv ${PREV_SG_SERVICE_NAME[$i]} >> $PREV_SG_SCRIPT_LOG_FILE 2>&1
                retval=$?
            fi
        fi

        if (( retval != 0 )) && ! sg_check_if_reconfig_has_failed
        then
            sg_log 0 "ERROR: Function sg_reconfigure_stop_services"
            sg_log 0 "ERROR: Failed to stop service"
            sg_log 999 "ERROR: Failed to halt service ${PREV_SG_SERVICE_NAME[$i]}"
            sg_set_reconfig_failure
            to_exit=1
        elif ((found==0)) && (( retval == 0 ))
        then
            sg_log 999 "SUCCESS: Successfully halted service ${PREV_SG_SERVICE_NAME[$i]}"
        fi
        ((i=i+1))
    done
}

##########################################################################
#
# sg_stop_services() is called while stopping a package (cmhaltpkg)
# or during rollback operation to recover from the package startup.
#
# Halt each service using cmhaltserv(1m).
#
##########################################################################

function sg_stop_services
{
    typeset -i retval=0

    sg_log 5 "sg_stop_services"

    for I in ${SG_SERVICE_NAME[@]}
    do
        sg_log 0 "Halting service $I"

        cmhaltserv $I >> $SG_SCRIPT_LOG_FILE 2>&1
        retval=$?

        #
        # sg_stop_services action is idempotent i.e.
        # if service is already halted cmhaltserv
        # prints the following message and returns success.
        #
        # cmhaltserv: Service name <name> is not running.
        #

        if (( retval != 0 ))
        then
            sg_log 0 "ERROR: Function sg_stop_resources"
            sg_log 0 "ERROR: Failed to halt service $I"
            to_exit=1
        fi
    done
}

################
# main routine
################

#
# Module script must be specified with three required entry points:
# start, stop, and validate.
#
# The variable to_exit indicates the success or failure of the entry 
# point.
#

typeset -i to_exit=0

sg_log 5 "sg_services"

case ${1} in
     start)      
           sg_start_services
           ;;
     stop)       
          if [[ "$SG_PACKAGE_DETACHED" = "yes" ]]
           then
              sg_log 5 "Calling cmserviced_halt_detached_services -D $SG_LOG_LEVEL" \
                       " -O $SG_SCRIPT_LOG_FILE $SG_PACKAGE 2>>$SG_SCRIPT_LOG_FILE"
              cmserviced_halt_detached_services -D $SG_LOG_LEVEL  \
                  -O $SG_SCRIPT_LOG_FILE $SG_PACKAGE 2>>$SG_SCRIPT_LOG_FILE
              retval=$?
              if (( retval != 0 ))
              then
                 sg_log 0 "ERROR: Function cmserviced_halt_detached_services: Failed"
                 to_exit=1
              fi
           else
              sg_stop_services
           fi
           ;;    
     validate)   
           sg_validate_services
           ;;
    reconfigure_stop)   
          sg_reconfigure_stop_services
          ;;
    reconfigure_start)   
          sg_reconfigure_start_services
          ;;
     *)
           sg_log 0 "INFO: Unknown operation - $1"
           ;;
esac

#
# Exit to indicate success/failure to the master_control_script.sh
#
exit $to_exit

