####################################################################
# (C) Copyright 2007-2008 Hewlett-Packard Development Company, L.P.
# @(#) Serviceguard MGPackageControl.pm
# @(#) Product Name                : HP Serviceguard
# @(#) Product Version             : A.12.10.00
# @(#) Patch Name                  : 
####################################################################

=pod

=head1 NAME

MGPackageControl - a class for Serviceguard packages

=cut

package MGPackage;

use strict;
use Carp;
use MGService;
use MGFileSystem;
use MGPackageDependency;
use MGPackageWeight;
use MGResource;
use MGUser;
use MGNetwork;
use POSIX qw(uname);
use File::Basename;

@MGPackage::notSupported = qw(
RAIDSTART
RAIDSTOP
RAIDTAB
MD
GFS
DATA_REP
DTC_NAME);

# HA server toolkits
@MGPackage::toolkit = qw(
EVFS_SCRIPT_EXTENSION
HA_NFS_SCRIPT_EXTENSION
HA_APP_SERVER);

@MGPackage::Extensions = qw(
ha_nfs_script_extension
ha_app_server);

@MGPackage::functionTable = qw(
activate_disk_group
activate_volume_group
add_ip_address
check_and_mount
check_dg
check_vxvm_vol_available
customer_defined_halt_cmds
customer_defined_run_cmds
deactivate_disk_group
deactivate_volume_group
disown_dtc
freeup_busy_mountpoint_and_mount_fs
get_ownership_dtc
ha_nfs_file_locks
halt_services
remove_ip_address
retry_print
start_resources
start_services
umount_fs
verify_ha_nfs
verify_physical_data_replication
wait_for_cvm_dg_vols_enabled
stop_resources
ps_tree
show_users
disown_dtc
dg_fuser
test_return
activation_check
check_gfs
lvm_sanity_check
vg_tag
verify_evfs
verify_ha_server
get_md
activate_md
deactivate_md
wait_for_diskgroup_enable
deactivate_dg
deactivate_dg_with_retries
persist_reservation
persist_release
sort_fs_directories
build_dev_list);

=pod

=item checkDTS 

=cut
sub checkDTS
{
    my ($self,$control) = @_;
 
    my $DRCheck;
    my $os_type  = (uname) [0];
    if ( $os_type eq "Linux") {
             my $sgsbin = getSGConfValue("SGSBIN");
             $DRCheck = $sgsbin ."/DRCheckDiskStatus";
    } else {
             $DRCheck = "/usr/sbin/DRCheckDiskStatus";
   } 

   # if DRCheckDiskStatus executable then look for env file
   if ( -x $DRCheck ) {
      # check if env file exits <pkgname>.*.env
        my $cntPath = dirname($control);
        my $envFile = $cntPath . "/" . $self->{name} . ".*.env";
        if ( -z $envFile) {
                print "Package ". $self->{name}. "is  a DTS, include DTS module before applying.\n"; 
        }
   }
}

=pod

=item checkVersion 

Check version in control script support versions A11.09, A11.12, A11.13, 
A11.14, A11.15, A11.16, A11.17,A11.18, A11.19
=cut

sub checkVersion
{
    my ($self, @lines) = @_;
    my $line = $lines[0];
    if ($line =~ /A.11.0[0-8].*/ || $line =~/A.10.*/) {
        return 1;
    }
    return 0;

}

=pod

=item checkFunctions 

=cut

sub checkFunctions
{
    my ($self, @lines) = @_;
    my $com = undef;
    my $linecnt=0;

    foreach my $line (@lines) {
        $linecnt++;
        next if ($line =~ /^\#/); # skip comments
        next if ($line =~ /^\s*$/); # skip blank lines
        if ($line =~ /^function /) {
             my ($value) = $line =~ /^\s*function\s*(\w.+)\s*$/;
             ($value,$com) = split(' ',$value);
             my $found = 0;
             foreach my $func (@{MGPackage::functionTable}) {
                 if ( $value eq $func) {
                      $found = 1;
                 }
             }
             if (!$found) {
                print "Warning: at line: ". $linecnt. " function " 
                      . $value . " is not a Serviceguard function.\n";
             }
        }
     }
}

=pod

=item parseControl 

=cut

sub parseControl
{
    my ($self, $pevs, @lines) = @_;
    my $lastService = undef;
    my $lastIpAddress = undef;
    my $name = undef;
    my $value = undef;
    my $com = undef;
    my $fileSystem = undef;
    my $saveDevice = undef;
    my $linecnt = 0;
    $self->{volumeGroups} = [];
    $self->{cvmDiskGroups} = [];
    $self->{vxvmDiskGroups} = [];
    $self->{multipleDevices} = [];
    $self->{fileSystems} = [];
    $self->{ipAddresses} = [];
    $self->{dtcs} = [];
    $self->{controlParams} = [];

    my $pkgtype = lc($self->getConfigParam("package_type"));
    my $bpval=0;

    foreach my $line (@lines) {
        $linecnt++;
        next if ($line =~ /^\#/); # skip comments
        next if ($line =~ /^\s*$/); # skip blank lines
        next if ($line =~ /^\s*PATH=/); # skip PATH define
        next if ($line =~ /^. /);
        last if ($line =~ /^function/); # stop when we've finished variables

        # VG[0]=vg01
        if ($line =~ /^\s*VG\[[0-9]+\]=\s*\"(\S+)\"/) {
            push(@{$self->{volumeGroups}}, $1);
        }
        # VXVM_DG[0]=dg01
        elsif ($line =~ /^\s*VXVM_DG\[[0-9]+\]=\s*\"(\S+)\"/) {
               push(@{$self->{vxvmDiskGroups}}, $1);
        }
        elsif ($line =~ /^s*LV\[[0-9]+\]=\"(\S+)\"/){
               if (!defined($saveDevice)){
                   $saveDevice = $1;
               }
        }
        elsif ($line =~ /^s*FS\[[0-9]+\]=\"(\S+)\"/){
                if (defined($fileSystem)) {
                    push(@{$self->{fileSystems}}, $fileSystem);
                }
                $fileSystem = MGFileSystem->new($1);
                $fileSystem->setDevice($saveDevice);
                undef $saveDevice;
        }
        elsif ($line =~ /^s*FS_TYPE\[[0-9]+\]=\"(\S.+)\"/){
                if (defined($fileSystem)) {
                     $fileSystem->setType($1);
                }
        }
        elsif ($line =~ /^s*FS_MOUNT_OPT\[[0-9]+\]=\"(\S.+)\"/){
                if (defined($fileSystem)) {
                    $fileSystem->setMountOptions($1);
                }
        }
        elsif ($line =~ /^s*FS_UMOUNT_OPT\[[0-9]+\]=\"(\S.+)\"/){
                if (defined($fileSystem)) {
                   $fileSystem->setUmountOptions($1);
                }
        }
        elsif ($line =~ /^s*FS_FSCK_OPT\[[0-9]+\]=\"(\S.+)\"/){
                if (defined($fileSystem)) {
                    $fileSystem->setFsckOptions($1);
                    push(@{$self->{fileSystems}}, $fileSystem);
                    undef $fileSystem;
                }
        }
        # IP[0]=192.10.25.12
        elsif ($line =~ /^\s*IP\[[0-9]+\]=\"(\S+)\"/) {
               if ($pkgtype eq "multi_node") {
                  print "WARNING : Multinode packages cannot use IP and SUBNET parameters. These parameters will not be migrated\n";
                  $bpval=1;
               }
               else {
                  $lastIpAddress = $1;
               }
        }
        # SUBNET[0]= 192.10.25.0
        elsif ($line =~ /^\s*SUBNET\[[0-9]+\]=\s*(\S+)/) {
               if ($pkgtype eq "multi_node") {
                  if (!$bpval) {
                     print "WARNING : Multinode packages cannot use IP and SUBNET parameters. These parameters will not be migrated\n";
                  }
               }
               else {
                  my $value = $1;
                  $value =~ s/[",\\]//g;
                  my $subnet  = undef;
                  $subnet = $self->getSubnet($value);
                  if ( !defined $subnet) {
                     $subnet = MGNetwork->new($value);
                     $self->addSubnet($subnet);
                  }
                  if (defined $lastIpAddress) {
                     $subnet->setIpAddress($lastIpAddress);
                     $lastIpAddress = undef;
                  }
               }
        }
        # SERVICE_NAME[0]="pkg1b"
        elsif ($line =~ /^\s*SERVICE_NAME\[[0-9]+\]=\"(\S+)\"/) {
                my $value = $1;
                # $value =~ s/[",\\]//g;
                $lastService = $self->getService($value);
                die "Error: Missing service $value in configuration ". 
                     "at line:$linecnt\n" . 
                     "Package migration failed.\n" if ! $lastService;
        }
        # SERVICE_CMD[1]="/usr/bin/X11/xload -display 192.10.25.54:0"
        elsif ($line =~ /^\s*SERVICE_CMD\[[0-9]+\]=\"(.+)\"/) {
               my $value = $1;
               if ( defined $lastService) {
                    $lastService->setCommand($value);
               }
        }
        # SERVICE_RESTART[1]="-r 2" remove -r and set -R to unlimited 
        elsif ($line =~ /^\s*SERVICE_RESTART\[[0-9]+\]=\"(.+)\"/) {
               my $rst = $1;
               if ( defined $lastService) {
                    if ($rst eq "-R" ) {
                        $lastService->setMaxRestarts("unlimited");
                    } else {
                        $rst =~ /-r\s*(\d)/;
                        $lastService->setMaxRestarts($1);
                    }
                    $lastService = undef;
               }
        }
       # DEFERRED_RESOURCE_NAME[0]="/a/resource"
        elsif ($line =~ /^\s*DEFERRED_RESOURCE_NAME\[[0-9]+\]=\s*(\S+)/) {
        #     This arry will be filled based on ascii file.
        }
        # DTC[0]=dtc_20
        elsif ($line =~ /^\s*DTC\[[0-9]+\]=\s*(\S+)/) {
            push(@{$self->{dtcs}}, $1);
        }
        # for example: VGCHANGE="vgchange -a y"
        elsif ($line =~ /^\s*(\w+)=\s*(.+)\s*$/) {
             $name = $1;
             $value = $2;
             ($value,$com) = split('#',$value);

             # pkg is using HA server toolkits variable then exit
             foreach my $toolvar (@{MGPackage::toolkit}) {
                         if ($name eq $toolvar) {
                             print "name=" . $name . "\n";
                             print "line=" . $line . "\n";
                             die "Error: This package is using the HA " .
                                 "server toolkits. This package can not be " .
                                 "converted to a modular package.\n" . 
                                 "Package migration failed.\n";
                         }
             }
 
             if ($name =~ /^VGCHANGE/i ||
                 $name =~ /^CVM_ACTIVATION_CMD/i ||
                 $name =~ /^VXVOL/i) {
                 $self->setControlParam($name,$value);
              }
              # remove extra quotes
              elsif ($name =~ /^KILL_PROCESSES_ACCESSING_RAW_DEVICES/i || 
                 $name =~ /^VXVM_DG_RETRY/i ||
                 $name =~ /^DEACTIVATION_RETRY_COUNT/i ||
                 $name =~ /^CONCURRENT_MOUNT_AND_UMOUNT_OPERATIONS/i ||
		 $name =~ /^ENABLE_THREADED_VGCHANGE/i ||
                 $name =~ /^CONCURRENT_VGCHANGE_OPERATIONS/i ||
                 $name =~ /^CONCURRENT_FSCK_OPERATIONS/i ||
                 $name =~ /^FS_MOUNT_RETRY_COUNT/i) { 

                 $value =~ /\"(.*)\"/;
                 $self->setControlParam($name,$1);
             }
             # set as packagename.log check if already defined in config
             elsif ($name =~ /log_file/i ) {
                    my $value = $self->getConfigParam("script_log_file");
                    if (!defined $value ) {
                        $name = "script_log_file";
                        $value = $self->{fileDir} . "/" . $self->{name} . 
                                 ".log" if (defined ($ENV{'SB_ROOT'}));
                        $self->setControlParam($name,$value);
                    }
             }
             elsif ($name =~ /^FS_UMOUNT_COUNT/i ) {
                 $name = "fs_umount_retry_count";
                 $value =~ /\"(.*)\"/;
                 $self->setControlParam($name,$1);
             } else {
                 # check for unsupported parameters
                 my $found = 0;
                 foreach my $attr (@{MGPackage::notSupported}) {
                         if ($name eq $attr) {
                              $found = 1;
                              print "Warning: at line: $linecnt " . $name . 
                                    " is a depreciated parameter.\n";

                         }
                  }
             # DO we need to check for values?
                 if (!$found && $pevs){ 
                     $self->setPevParam($name,$value);
                 }
              }
        }
    } # each line

    if (defined $fileSystem) {
                push(@{$self->{fileSystems}}, $fileSystem);
    }
}

=pod

=item readControl 

=cut

sub readControl
{
   my ($self,$name,$external,$pevs) = @_;
   die "Error: ". $name . " is not a legacy package, since it does not have a\n" . 
        "control script. It can not be converted.\n" . 
        "Package migration failed\n" if ! $self->{controlPath}; 

   my $control = $self->{controlPath};
   open(MYINPUTFILE, $control ) or 
        die "Error: Failed to open $control\n" . "Package migration failed.\n";

   my (@lines) = <MYINPUTFILE>;
   if ( $self->checkVersion(@lines) ) {
        die "Error: Package Version is not supported to migrate. It must be " . 
             "at least A.11.09.XX,or A.11.12.XX - A.11.19.XX \n" .
             "Package migration failed. \n";
   }
   # preprocess control script to get the parameters in expected format
   my $myCmd = $self->getConf . "/" . "preprocess.sh " . $control . " |";
 
   open (MYCMD, $myCmd) or
         die "Error: Failed to invoke: $myCmd" .
         "Package migration failed.\n";
   my (@procControl) = <MYCMD>;
   $self->parseControl($pevs, @procControl);

   $self->checkFunctions(@lines);
   $self->readCustomer(@lines) if (defined($external));
   close(MYINPUTFILE);

   $self->checkDTS($control);

}

=pod

=item readCustomer 

=cut

sub readCustomer
{
   my ($self,@lines) = @_;
   my $txtcomment="\t" . "# if failure log error and return failure";
   my $txt1="\t" . "# if [[ $? != 0 ]]; then" .
             "\n\t\t" . "# sg_log 0  " . '"external script start error"'. "\n";

   my $txt2="\t" . "# if [[ $? != 0 ]]; then" .
            "\n\t\t" . "# sg_log 0  " . '"external script stop error"'. "\n";

   my $txt3="\t\t" . "# return 1;" .  "\n\t" . "# fi" . "\n";


   my $cnt = 0;
   my $found = 0;
   # skip down to function customer_defined_run_cmds
   while (($cnt < @lines) && ! $found) {
          if ( $lines[$cnt++] =~ /customer_defined_run_cmds/) {
               $found = 1;
          }
   }
  
   if (!$found ) {
       print "No customer_defined_run_cmd found in control script\n";
       return;
   }
   $found = 0; 
   # read past {, save function info
   $cnt++;
   while (($cnt < @lines) && ! $found) {
           if ($lines[$cnt] =~ /^\}/) {
                   $found = 1;
           } else {
              if (($lines[$cnt] =~ /test_return 51/ )) {
                   push(@{$self->{start_script}},$txtcomment);
                   push(@{$self->{start_script}},$txt1);
                   push(@{$self->{start_script}},$txt3);
              } else {
                   push(@{$self->{start_script}},$lines[$cnt]);
              }
           }
           $cnt++;
   }
   $found = 0; 
   # save code into array for stop
   while (($cnt < @lines) && ! $found) {
           if ($lines[$cnt] =~ /^function customer_defined_halt_cmds/) {
               $found = 1;
           } 
           $cnt++;
   }
   # read past {,and save halt function info
   $cnt++;
   $found = 0; 
   while (($cnt < @lines) && ! $found) {
         if ($lines[$cnt] =~ /^\}/) {
                   $found = 1;
          } else {
             if (($lines[$cnt] =~ /test_return 52/ )) {
                  push(@{$self->{start_script}},$txtcomment);
                  push(@{$self->{stop_script}},$txt2);
                  push(@{$self->{stop_script}},$txt3);
             } else {
                 push(@{$self->{stop_script}},$lines[$cnt]);
             }
          }
          $cnt++;
   }
   $found = 0;
   while (($cnt < @lines) && ! $found) {
           if ($lines[$cnt] =~ /^function /) {
               $found = 2;
           } 
           elsif ($lines[$cnt] =~ /# END OF CUSTOMER DEFINED FUNCTIONS/) {
               $found = 1; 
           } else {
               $cnt++;
           }
   } 
   return if ($found < 2 ); 

   # read to END of CUSTOMER DEFINED FUNCTIONS 
   $found = 0;
   while (($cnt < @lines) && ! $found) {
               if ($lines[$cnt] =~ /# END OF CUSTOMER DEFINED FUNCTIONS/) {
                   $found = 1;
               } else {
                   push(@{$self->{users_functions}},$lines[$cnt]);
               }
               $cnt++;
    }
}


1; # for perl include
